#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: iam_role
version_added: 1.0.0
version_added_collection: community.aws
short_description: Manage AWS IAM roles
description:
  - Manage AWS IAM roles.
author:
  - "Rob White (@wimnat)"
options:
  path:
    description:
      - The path of the role.
      - For more information about IAM paths, see the AWS IAM identifiers documentation
        U(https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_identifiers.html).
      - Updating the path on an existing role is not currently supported and will result in a
        warning.
      - O(path_prefix) and O(prefix) were added as aliases in release 7.2.0.
    type: str
    aliases: ["prefix", "path_prefix"]
  name:
    description:
      - The name of the role.
      - >-
        Note: Role names are unique within an account.  Paths (O(path)) do B(not) affect
        the uniqueness requirements of O(name).  For example it is not permitted to have both
        C(/Path1/MyRole) and C(/Path2/MyRole) in the same account.
      - O(role_name) was added as an alias in release 7.2.0.
    required: true
    type: str
    aliases: ["role_name"]
  description:
    description:
      - Provides a description of the role.
    type: str
  boundary:
    description:
      - The ARN of an IAM managed policy to use to restrict the permissions this role can pass on to IAM roles/users that it creates.
      - Boundaries cannot be set on Instance Profiles, as such if this option is specified then O(create_instance_profile) must be V(false).
      - This is intended for roles/users that have permissions to create new IAM objects.
      - For more information on boundaries, see U(https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_boundaries.html).
    aliases: [boundary_policy_arn]
    type: str
  assume_role_policy_document:
    description:
      - The trust relationship policy document that grants an entity permission to assume the role.
      - This parameter is required when O(state=present).
    type: json
  managed_policies:
    description:
      - A list of managed policy ARNs, or friendly names.
      - To remove all policies set O(purge_policies=true) and O(managed_policies=[]).
      - To embed an inline policy, use M(amazon.aws.iam_policy).
    aliases: ['managed_policy']
    type: list
    elements: str
  max_session_duration:
    description:
      - The maximum duration (in seconds) of a session when assuming the role.
      - Valid values are between 1 and 12 hours (3600 and 43200 seconds).
    type: int
  purge_policies:
    description:
      - When O(purge_policies=true) any managed policies not listed in O(managed_policies) will be detatched.
    type: bool
    aliases: ['purge_policy', 'purge_managed_policies']
    default: true
  state:
    description:
      - Create or remove the IAM role.
    default: present
    choices: [ present, absent ]
    type: str
  create_instance_profile:
    description:
      - If no IAM instance profile with the same O(name) exists, setting O(create_instance_profile=True)
        will create an IAM instance profile along with the role.
      - This option has been deprecated and will be removed in a release after 2026-05-01.  The
        M(amazon.aws.iam_instance_profile) module can be used to manage instance profiles.
      - Defaults to V(True)
    type: bool
  delete_instance_profile:
    description:
      - When O(delete_instance_profile=true) and O(state=absent) deleting a role will also delete an
        instance profile with the same O(name) as the role, but only if the instance profile is
        associated with the role.
      - Only applies when O(state=absent).
      - This option has been deprecated and will be removed in a release after 2026-05-01.  The
        M(amazon.aws.iam_instance_profile) module can be used to manage instance profiles.
      - Defaults to V(False)
    type: bool
  wait_timeout:
    description:
      - How long (in seconds) to wait for creation / update to complete.
    default: 120
    type: int
  wait:
    description:
      - When O(wait=True) the module will wait for up to O(wait_timeout) seconds
        for IAM role creation before returning.
    default: True
    type: bool
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.tags
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Create a role with description and tags
  amazon.aws.iam_role:
    name: mynewrole
    assume_role_policy_document: "{{ lookup('file','policy.json') }}"
    description: This is My New Role
    tags:
      env: dev

- name: "Create a role and attach a managed policy called 'PowerUserAccess'"
  amazon.aws.iam_role:
    name: mynewrole
    assume_role_policy_document: "{{ lookup('file','policy.json') }}"
    managed_policies:
      - arn:aws:iam::aws:policy/PowerUserAccess

- name: Keep the role created above but remove all managed policies
  amazon.aws.iam_role:
    name: mynewrole
    assume_role_policy_document: "{{ lookup('file','policy.json') }}"
    managed_policies: []

- name: Delete the role
  amazon.aws.iam_role:
    name: mynewrole
    assume_role_policy_document: "{{ lookup('file', 'policy.json') }}"
    state: absent
"""

RETURN = r"""
iam_role:
    description: Dictionary containing the IAM Role data.
    returned: success
    type: complex
    contains:
        path:
            description: The path to the role.
            type: str
            returned: always
            sample: /
        role_name:
            description: The friendly name that identifies the role.
            type: str
            returned: always
            sample: myrole
        role_id:
            description: The stable and unique string identifying the role.
            type: str
            returned: always
            sample: ABCDEFF4EZ4ABCDEFV4ZC
        arn:
            description: The Amazon Resource Name (ARN) specifying the role.
            type: str
            returned: always
            sample: "arn:aws:iam::1234567890:role/mynewrole"
        create_date:
            description: The date and time, in ISO 8601 date-time format, when the role was created.
            type: str
            returned: always
            sample: "2016-08-14T04:36:28+00:00"
        assume_role_policy_document:
            description:
              - The policy that grants an entity permission to assume the role.
              - |
                Note: the case of keys in this dictionary are no longer converted from CamelCase to
                snake_case. This behaviour changed in release 8.0.0.
            type: dict
            returned: always
            sample: {
                        'statement': [
                            {
                                'action': 'sts:AssumeRole',
                                'effect': 'Allow',
                                'principal': {
                                    'service': 'ec2.amazonaws.com'
                                },
                                'sid': ''
                            }
                        ],
                        'version': '2012-10-17'
                    }
        assume_role_policy_document_raw:
            description:
              - |
                Note: this return value has been deprecated and will be removed in a release after
                2026-05-01. RV(iam_role.assume_role_policy_document) and RV(iam_role.assume_role_policy_document_raw)
                now use the same format.
            type: dict
            returned: always
            sample: {
                        'statement': [
                            {
                                'action': 'sts:AssumeRole',
                                'effect': 'Allow',
                                'principal': {
                                    'service': 'ec2.amazonaws.com'
                                },
                                'sid': ''
                            }
                        ],
                        'version': '2012-10-17'
                    }
            version_added: 5.3.0
        attached_policies:
            description: A list of dicts containing the name and ARN of the managed IAM policies attached to the role.
            type: list
            elements: dict
            returned: always
            sample: [
                {
                    'policy_arn': 'arn:aws:iam::aws:policy/PowerUserAccess',
                    'policy_name': 'PowerUserAccess'
                }
            ]
            contains:
                policy_arn:
                    description: The Amazon Resource Name (ARN) specifying the managed policy.
                    type: str
                    sample: "arn:aws:iam::123456789012:policy/test_policy"
                policy_name:
                    description: The friendly name that identifies the policy.
                    type: str
                    sample: test_policy
        description:
            description: A description of the role.
            type: str
            returned: always
            sample: "This is My New Role"
        max_session_duration:
            description: The maximum duration (in seconds) of a session when assuming the role.
            type: int
            returned: always
            sample: 3600
        role_last_used:
            description: Contains information about the last time that an IAM role was used.
            type: dict
            returned: always
            sample: {
                        "last_used_date": "2023-11-22T21:54:29+00:00",
                        "region": "us-east-2"
                    }
            contains:
                last_used_date:
                    description: The date and time, in  ISO 8601 date-time format that the role was last used.
                    type: str
                    returned: always
                region:
                    description: The name of the Amazon Web Services Region in which the role was last used.
                    type: str
                    returned: always
        tags:
            description: role tags
            type: dict
            returned: always
            sample: '{"Env": "Prod"}'
"""

import json

from ansible_collections.amazon.aws.plugins.module_utils.arn import validate_aws_arn
from ansible_collections.amazon.aws.plugins.module_utils.iam import AnsibleIAMError
from ansible_collections.amazon.aws.plugins.module_utils.iam import IAMErrorHandler
from ansible_collections.amazon.aws.plugins.module_utils.iam import add_role_to_iam_instance_profile
from ansible_collections.amazon.aws.plugins.module_utils.iam import convert_managed_policy_names_to_arns
from ansible_collections.amazon.aws.plugins.module_utils.iam import create_iam_instance_profile
from ansible_collections.amazon.aws.plugins.module_utils.iam import delete_iam_instance_profile
from ansible_collections.amazon.aws.plugins.module_utils.iam import get_iam_role
from ansible_collections.amazon.aws.plugins.module_utils.iam import list_iam_instance_profiles
from ansible_collections.amazon.aws.plugins.module_utils.iam import list_iam_role_attached_policies
from ansible_collections.amazon.aws.plugins.module_utils.iam import normalize_iam_role
from ansible_collections.amazon.aws.plugins.module_utils.iam import remove_role_from_iam_instance_profile
from ansible_collections.amazon.aws.plugins.module_utils.iam import validate_iam_identifiers
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.policy import compare_policies
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import ansible_dict_to_boto3_tag_list
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.tagging import compare_aws_tags


class AnsibleIAMAlreadyExistsError(AnsibleIAMError):
    pass


@IAMErrorHandler.common_error_handler("wait for role creation")
def wait_iam_exists(client, check_mode, role_name, wait, wait_timeout):
    if check_mode or wait:
        return

    delay = min(wait_timeout, 5)
    max_attempts = wait_timeout // delay

    waiter = client.get_waiter("role_exists")
    waiter.wait(
        WaiterConfig={"Delay": delay, "MaxAttempts": max_attempts},
        RoleName=role_name,
    )


def attach_policies(client, check_mode, policies_to_attach, role_name):
    if not policies_to_attach:
        return False
    if check_mode:
        return True

    for policy_arn in policies_to_attach:
        IAMErrorHandler.common_error_handler(f"attach policy {policy_arn} to role")(client.attach_role_policy)(
            RoleName=role_name, PolicyArn=policy_arn, aws_retry=True
        )
    return True


def remove_policies(client, check_mode, policies_to_remove, role_name):
    if not policies_to_remove:
        return False
    if check_mode:
        return True

    for policy in policies_to_remove:
        IAMErrorHandler.deletion_error_handler(f"detach policy {policy} from role")(client.detach_role_policy)(
            RoleName=role_name, PolicyArn=policy, aws_retry=True
        )
    return True


def remove_inline_policies(client, role_name):
    current_inline_policies = get_inline_policy_list(client, role_name)
    for policy in current_inline_policies:
        IAMErrorHandler.deletion_error_handler(f"delete policy {policy} embedded in role")(client.delete_role_policy)(
            RoleName=role_name, PolicyName=policy, aws_retry=True
        )


def generate_create_params(module):
    params = dict()
    params["Path"] = module.params.get("path") or "/"
    params["RoleName"] = module.params.get("name")
    params["AssumeRolePolicyDocument"] = module.params.get("assume_role_policy_document")
    if module.params.get("description") is not None:
        params["Description"] = module.params.get("description")
    if module.params.get("max_session_duration") is not None:
        params["MaxSessionDuration"] = module.params.get("max_session_duration")
    if module.params.get("boundary") is not None:
        params["PermissionsBoundary"] = module.params.get("boundary")
    if module.params.get("tags") is not None:
        params["Tags"] = ansible_dict_to_boto3_tag_list(module.params.get("tags"))

    return params


@IAMErrorHandler.common_error_handler("create role")
def create_basic_role(module, client):
    """
    Perform the Role creation.
    Assumes tests for the role existing have already been performed.
    """
    if module.check_mode:
        module.exit_json(changed=True)

    params = generate_create_params(module)
    role = client.create_role(aws_retry=True, **params)
    # 'Description' is documented as a key of the role returned by create_role
    # but appears to be an AWS bug (the value is not returned using the AWS CLI either).
    # Get the role after creating it.
    # nb. doesn't use get_iam_role because we need to retry if the Role isn't there
    role = _get_role_with_backoff(client, params["RoleName"])

    return role


@IAMErrorHandler.common_error_handler("update assume role policy for role")
def update_role_assumed_policy(client, check_mode, role_name, target_assumed_policy, current_assumed_policy):
    # Check Assumed Policy document
    if target_assumed_policy is None or not compare_policies(current_assumed_policy, json.loads(target_assumed_policy)):
        return False
    if check_mode:
        return True

    client.update_assume_role_policy(RoleName=role_name, PolicyDocument=target_assumed_policy, aws_retry=True)
    return True


@IAMErrorHandler.common_error_handler("update description for role")
def update_role_description(client, check_mode, role_name, target_description, current_description):
    # Check Description update
    if target_description is None or current_description == target_description:
        return False
    if check_mode:
        return True

    client.update_role(RoleName=role_name, Description=target_description, aws_retry=True)
    return True


@IAMErrorHandler.common_error_handler("update maximum session duration for role")
def update_role_max_session_duration(client, check_mode, role_name, target_duration, current_duration):
    # Check MaxSessionDuration update
    if target_duration is None or current_duration == target_duration:
        return False
    if check_mode:
        return True

    client.update_role(RoleName=role_name, MaxSessionDuration=target_duration, aws_retry=True)
    return True


@IAMErrorHandler.common_error_handler("update permission boundary for role")
def _put_role_permissions_boundary(client, **params):
    client.put_role_permissions_boundary(aws_retry=True, **params)


@IAMErrorHandler.deletion_error_handler("remove permission boundary from role")
def _delete_role_permissions_boundary(client, **params):
    client.delete_role_permissions_boundary(**params)


def update_role_permissions_boundary(client, check_mode, role_name, permissions_boundary, current_permissions_boundary):
    # Check PermissionsBoundary
    if permissions_boundary is None or permissions_boundary == current_permissions_boundary:
        return False
    if check_mode:
        return True

    if permissions_boundary == "":
        _delete_role_permissions_boundary(client, RoleName=role_name)
    else:
        _put_role_permissions_boundary(client, RoleName=role_name, PermissionsBoundary=permissions_boundary)
    return True


def update_managed_policies(client, check_mode, role_name, managed_policies, purge_policies):
    # Check Managed Policies
    if managed_policies is None:
        return False

    # Get list of current attached managed policies
    current_attached_policies = list_iam_role_attached_policies(client, role_name)
    current_attached_policies_arn_list = [policy["PolicyArn"] for policy in current_attached_policies]

    if len(managed_policies) == 1 and managed_policies[0] is None:
        managed_policies = []

    policies_to_remove = set(current_attached_policies_arn_list) - set(managed_policies)
    policies_to_remove = policies_to_remove if purge_policies else []
    policies_to_attach = set(managed_policies) - set(current_attached_policies_arn_list)

    changed = False
    if purge_policies and policies_to_remove:
        if check_mode:
            return True
        else:
            changed |= remove_policies(client, check_mode, policies_to_remove, role_name)

    if policies_to_attach:
        if check_mode:
            return True
        else:
            changed |= attach_policies(client, check_mode, policies_to_attach, role_name)

    return changed


def update_basic_role(module, client, role_name, role):
    check_mode = module.check_mode
    assumed_policy = module.params.get("assume_role_policy_document")
    description = module.params.get("description")
    duration = module.params.get("max_session_duration")
    path = module.params.get("path")
    permissions_boundary = module.params.get("boundary")
    purge_tags = module.params.get("purge_tags")
    tags = module.params.get("tags")

    # current attributes
    current_assumed_policy = role.get("AssumeRolePolicyDocument")
    current_description = role.get("Description")
    current_duration = role.get("MaxSessionDuration")
    current_permissions_boundary = role.get("PermissionsBoundary", {}).get("PermissionsBoundaryArn", "")
    current_tags = role.get("Tags", [])

    # As of botocore 1.34.3, the APIs don't support updating the Name or Path
    if update_role_path(client, check_mode, role, path):
        module.warn(
            "iam_role doesn't support updating the path: " f"current path '{role.get('Path')}', requested path '{path}'"
        )

    changed = False

    # Update attributes
    changed |= update_role_tags(client, check_mode, role_name, tags, purge_tags, current_tags)
    changed |= update_role_assumed_policy(client, check_mode, role_name, assumed_policy, current_assumed_policy)
    changed |= update_role_description(client, check_mode, role_name, description, current_description)
    changed |= update_role_max_session_duration(client, check_mode, role_name, duration, current_duration)
    changed |= update_role_permissions_boundary(
        client, check_mode, role_name, permissions_boundary, current_permissions_boundary
    )

    return changed


def create_or_update_role(module, client, role_name, create_instance_profile):
    check_mode = module.check_mode
    wait = module.params.get("wait")
    wait_timeout = module.params.get("wait_timeout")
    path = module.params.get("path")
    purge_policies = module.params.get("purge_policies")
    managed_policies = module.params.get("managed_policies")
    if managed_policies:
        # Attempt to list the policies early so we don't leave things behind if we can't find them.
        managed_policies = convert_managed_policy_names_to_arns(client, managed_policies)

    changed = False

    # Get role
    role = get_iam_role(client, role_name)

    # If role is None, create it
    if role is None:
        role = create_basic_role(module, client)
        wait_iam_exists(client, check_mode, role_name, wait, wait_timeout)
        changed = True
    else:
        changed = update_basic_role(module, client, role_name, role)
        wait_iam_exists(client, check_mode, role_name, wait, wait_timeout)

    if create_instance_profile:
        try:
            changed |= create_instance_profiles(client, check_mode, role_name, path)
            wait_iam_exists(client, check_mode, role_name, wait, wait_timeout)
        except AnsibleIAMAlreadyExistsError as e:
            module.warn(f"profile {role_name} already exists and will not be updated")

    changed |= update_managed_policies(client, module.check_mode, role_name, managed_policies, purge_policies)
    wait_iam_exists(client, check_mode, role_name, wait, wait_timeout)

    # Get the role again
    role = get_iam_role(client, role_name)
    role["AttachedPolicies"] = list_iam_role_attached_policies(client, role_name)
    camel_role = normalize_iam_role(role, _v7_compat=True)

    module.exit_json(changed=changed, iam_role=camel_role)


def create_instance_profiles(client, check_mode, role_name, path):
    # Fetch existing Profiles
    role_profiles = list_iam_instance_profiles(client, role=role_name)
    # Profile already exists
    if any(p["InstanceProfileName"] == role_name for p in role_profiles):
        return False

    named_profile = list_iam_instance_profiles(client, name=role_name)
    if named_profile:
        raise AnsibleIAMAlreadyExistsError(f"profile {role_name} already exists")

    if check_mode:
        return True

    path = path or "/"
    # Make sure an instance profile is created
    create_iam_instance_profile(client, role_name, path, {})
    add_role_to_iam_instance_profile(client, role_name, role_name)

    return True


def remove_instance_profiles(client, check_mode, role_name, delete_instance_profile):
    """Removes the role from instance profiles and deletes the instance profile if
    delete_instance_profile is set
    """

    instance_profiles = list_iam_instance_profiles(client, role=role_name)
    if not instance_profiles:
        return False
    if check_mode:
        return True

    # Remove the role from the instance profile(s)
    for profile in instance_profiles:
        profile_name = profile["InstanceProfileName"]
        remove_role_from_iam_instance_profile(client, profile_name, role_name)
        if not delete_instance_profile:
            continue
        # Delete the instance profile if the role and profile names match
        if profile_name == role_name:
            delete_iam_instance_profile(client, profile_name)


@IAMErrorHandler.deletion_error_handler("delete role")
def destroy_role(client, check_mode, role_name, delete_profiles):
    role = get_iam_role(client, role_name)

    if role is None:
        return False

    if check_mode:
        return True

    # Before we try to delete the role we need to remove any
    # - attached instance profiles
    # - attached managed policies
    # - embedded inline policies
    remove_instance_profiles(client, check_mode, role_name, delete_profiles)
    update_managed_policies(client, check_mode, role_name, [], True)
    remove_inline_policies(client, role_name)

    client.delete_role(aws_retry=True, RoleName=role_name)
    return True


@IAMErrorHandler.common_error_handler("get role")
@AWSRetry.jittered_backoff(catch_extra_error_codes=["NoSuchEntity"])
def _get_role_with_backoff(client, name):
    client.get_role(RoleName=name)["Role"]


@IAMErrorHandler.list_error_handler("list attached inline policies for role")
def get_inline_policy_list(client, name):
    return client.list_role_policies(RoleName=name, aws_retry=True)["PolicyNames"]


def update_role_path(client, check_mode, role, path):
    if path is None:
        return False
    if path == role.get("Path"):
        return False
    if check_mode:
        return True

    # Not currently supported by the APIs
    pass
    return True


@IAMErrorHandler.common_error_handler("set tags for role")
def update_role_tags(client, check_mode, role_name, new_tags, purge_tags, existing_tags):
    if new_tags is None:
        return False
    existing_tags = boto3_tag_list_to_ansible_dict(existing_tags)

    tags_to_add, tags_to_remove = compare_aws_tags(existing_tags, new_tags, purge_tags=purge_tags)
    if not tags_to_remove and not tags_to_add:
        return False
    if check_mode:
        return True

    if tags_to_remove:
        client.untag_role(RoleName=role_name, TagKeys=tags_to_remove, aws_retry=True)
    if tags_to_add:
        client.tag_role(RoleName=role_name, Tags=ansible_dict_to_boto3_tag_list(tags_to_add), aws_retry=True)

    return True


def validate_params(module):
    if module.params.get("boundary"):
        # We need to handle both None and True
        if module.params.get("create_instance_profile") is not False:
            module.fail_json(msg="When using a boundary policy, `create_instance_profile` must be set to `false`.")
        if not validate_aws_arn(module.params.get("boundary"), service="iam"):
            module.fail_json(msg="Boundary policy must be an ARN")
    if module.params.get("max_session_duration"):
        max_session_duration = module.params.get("max_session_duration")
        if max_session_duration < 3600 or max_session_duration > 43200:
            module.fail_json(msg="max_session_duration must be between 1 and 12 hours (3600 and 43200 seconds)")

    identifier_problem = validate_iam_identifiers(
        "role", name=module.params.get("name"), path=module.params.get("path")
    )
    if identifier_problem:
        module.fail_json(msg=identifier_problem)


def main():
    argument_spec = dict(
        name=dict(type="str", aliases=["role_name"], required=True),
        path=dict(type="str", aliases=["path_prefix", "prefix"]),
        assume_role_policy_document=dict(type="json"),
        managed_policies=dict(type="list", aliases=["managed_policy"], elements="str"),
        max_session_duration=dict(type="int"),
        state=dict(type="str", choices=["present", "absent"], default="present"),
        description=dict(type="str"),
        boundary=dict(type="str", aliases=["boundary_policy_arn"]),
        create_instance_profile=dict(type="bool"),
        delete_instance_profile=dict(type="bool"),
        purge_policies=dict(default=True, type="bool", aliases=["purge_policy", "purge_managed_policies"]),
        tags=dict(type="dict", aliases=["resource_tags"]),
        purge_tags=dict(type="bool", default=True),
        wait=dict(type="bool", default=True),
        wait_timeout=dict(default=120, type="int"),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        required_if=[("state", "present", ["assume_role_policy_document"])],
        supports_check_mode=True,
    )

    module.deprecate(
        "In a release after 2026-05-01 iam_role.assume_role_policy_document_raw "
        "will no longer be returned.  Since release 8.0.0 assume_role_policy_document "
        "has been returned with the same format as iam_role.assume_role_policy_document_raw",
        date="2026-05-01",
        collection_name="amazon.aws",
    )
    if module.params.get("create_instance_profile") is None:
        module.deprecate(
            "In a release after 2026-05-01 the 'create_instance_profile' option will be removed. "
            "The amazon.aws.iam_instance_profile module can be used to manage instance profiles instead.",
            date="2026-05-01",
            collection_name="amazon.aws",
        )
    if module.params.get("delete_instance_profile") is None:
        module.deprecate(
            "In a release after 2026-05-01 the 'delete_instance_profile' option will be removed. "
            "The amazon.aws.iam_instance_profile module can be used to manage and delete instance "
            "profiles instead.",
            date="2026-05-01",
            collection_name="amazon.aws",
        )

    validate_params(module)

    client = module.client("iam", retry_decorator=AWSRetry.jittered_backoff())

    state = module.params.get("state")
    role_name = module.params.get("name")

    create_profile = module.params.get("create_instance_profile")
    create_profile = True if create_profile is None else create_profile
    delete_profile = module.params.get("delete_instance_profile") or False

    try:
        if state == "present":
            create_or_update_role(module, client, role_name, create_profile)
        elif state == "absent":
            changed = destroy_role(client, module.check_mode, role_name, delete_profile)
            module.exit_json(changed=changed)
    except AnsibleIAMError as e:
        module.fail_json_aws_error(e)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: deployment_info
short_description: Information module for Deployment
description:
- Get all Deployment.
- This API allows the client to pull the deployment information.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module_info
author: Rafael Campos (@racampos)
options:
  timeout:
    description:
    - How long to wait for the server to send data before giving up.
    type: int
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
seealso:
- name: Cisco ISE documentation for PullDeploymentInfo
  description: Complete reference of the PullDeploymentInfo API.
  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!deploymentinfo
notes:
  - SDK Method used are
    pull_deployment_info.PullDeploymentInfo.get_deployment_info,

  - Paths used are
    get /ers/config/deploymentinfo/getAllInfo,

"""

EXAMPLES = r"""
- name: Get all Deployment
  cisco.ise.deployment_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
  register: result

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "networkAccessInfo": {
        "deploymentID": "string",
        "isCsnEnabled": true,
        "nodeList": {
          "nodeAndScope": [
            {}
          ]
        },
        "sdaVNs": [],
        "trustSecControl": "string",
        "radius3RdParty": []
      },
      "profilerInfo": {
        "deploymentID": "string",
        "nodeList": {
          "node": [
            {
              "onlineSubscriptionEnabled": true,
              "lastAppliedFeedDateTime": "string",
              "scope": "string",
              "profiles": [
                {
                  "profile": [],
                  "customProfilesCount": 0,
                  "endpointTypes": "string",
                  "totalProfilesCount": 0,
                  "uniqueEndpointsCount": 0,
                  "unknownEndpointsCount": 0,
                  "totalEndpointsCount": 0,
                  "unknownEndpointsPercentage": 0
                }
              ]
            }
          ]
        }
      },
      "deploymentInfo": {
        "deploymentID": "string",
        "versionHistoryInfo": [
          {
            "opType": "string",
            "mainVersion": "string",
            "epochTime": 0
          }
        ],
        "nodeList": {
          "nodeAndNodeCountAndCountInfo": [
            {
              "name": "string",
              "value": {},
              "declaredType": "string",
              "scope": "string",
              "nil": true,
              "globalScope": true,
              "typeSubstituted": true
            }
          ]
        },
        "fipsstatus": "string"
      },
      "nadInfo": {
        "nodeList": {
          "nodeAndScope": [
            {}
          ]
        },
        "nadcountInfo": {
          "totalActiveNADCount": 0
        }
      },
      "mdmInfo": {
        "activeMdmServersCount": "string",
        "activeDesktopMdmServersCount": "string",
        "activeMobileMdmServersCount": "string",
        "deploymentID": "string",
        "nodeList": {
          "nodeAndScope": [
            {}
          ]
        }
      },
      "licensesInfo": {
        "deploymentID": "string",
        "nodeList": {
          "node": [
            {}
          ]
        }
      },
      "postureInfo": {
        "content": [
          {
            "name": "string",
            "value": {},
            "declaredType": "string",
            "scope": "string",
            "nil": true,
            "globalScope": true,
            "typeSubstituted": true
          }
        ]
      },
      "kongInfo": {
        "deploymentID": "string",
        "nodeList": {
          "node": [
            {
              "sn": "string",
              "service": [
                {
                  "serviceName": "string",
                  "route": [
                    {
                      "routeName": "string",
                      "httpCount": {},
                      "latencyCount": {},
                      "latencySum": {}
                    }
                  ]
                }
              ]
            }
          ]
        }
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: subscriber
short_description: Resource module for Subscriber
description:
- Manage operations create, update and delete of the resource Subscriber.
version_added: '2.8.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  enabled:
    description: Subscriber is enabled or not.
    type: bool
  friendlyName:
    description: Friendly name for the subscriber.
    type: str
  identityGroups:
    description: Identity Group(s). With more than one idGroups it needs to be comma
      seperated.
    type: str
  imeis:
    description: IMEI to be attached to the subscriber.
    type: str
  imsi:
    description: IMSI for Subscriber.
    type: str
  ki:
    description: KI.
    type: str
  opc:
    description: OPC.
    type: str
  subscriberId:
    description: SubscriberId path parameter. Unique id for a subscriber object.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    subscriber.Subscriber.create_subscriber,
    subscriber.Subscriber.delete_subscriber,
    subscriber.Subscriber.update_subscriber,

  - Paths used are
    post /api/v1/fiveg/subscriber,
    delete /api/v1/fiveg/subscriber/{subscriberId},
    put /api/v1/fiveg/subscriber/{subscriberId},

"""

EXAMPLES = r"""
- name: Create
  cisco.ise.subscriber:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    enabled: true
    friendlyName: string
    identityGroups: string
    imeis: string
    imsi: string
    ki: string
    opc: string

- name: Update by id
  cisco.ise.subscriber:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    enabled: true
    friendlyName: string
    identityGroups: string
    imeis: string
    ki: string
    opc: string
    subscriberId: string

- name: Delete by id
  cisco.ise.subscriber:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: absent
    subscriberId: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "createTime": "string",
      "enabled": true,
      "friendlyName": "string",
      "id": "string",
      "identityGroups": "string",
      "imeis": "string",
      "imsi": "string",
      "ki": "string",
      "link": {
        "href": "string",
        "rel": "string",
        "type": "string"
      },
      "opc": "string",
      "updateTime": "string"
    }

ise_update_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "createTime": "string",
        "enabled": true,
        "friendlyName": "string",
        "id": "string",
        "identityGroups": "string",
        "imeis": "string",
        "imsi": "string",
        "ki": "string",
        "link": {
          "href": "string",
          "rel": "string",
          "type": "string"
        },
        "opc": "string",
        "updateTime": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: swim_import_via_url_v1
short_description: Resource module for Swim Import Via Url V1
description:
- Manage operation create of the resource Swim Import Via Url V1.
- >
   Fetches a software image from remote file system using URL for HTTP/FTP and uploads to DNA Center. Supported image
   files extensions are bin, img, tar, smu, pie, aes, iso, ova, tar_gz and qcow2.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  payload:
    description: Swim Import Via Url's payload.
    elements: dict
    suboptions:
      applicationType:
        description: Swim Import Via Url's applicationType.
        type: str
      imageFamily:
        description: Swim Import Via Url's imageFamily.
        type: str
      sourceURL:
        description: Swim Import Via Url's sourceURL.
        type: str
      thirdParty:
        description: ThirdParty flag.
        type: bool
      vendor:
        description: Swim Import Via Url's vendor.
        type: str
    type: list
  scheduleAt:
    description: ScheduleAt query parameter. Epoch Time (The number of milli-seconds
      since January 1 1970 UTC) at which the distribution should be scheduled (Optional).
    type: str
  scheduleDesc:
    description: ScheduleDesc query parameter. Custom Description (Optional).
    type: str
  scheduleOrigin:
    description: ScheduleOrigin query parameter. Originator of this call (Optional).
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Software Image Management (SWIM) ImportSoftwareImageViaURLV1
  description: Complete reference of the ImportSoftwareImageViaURLV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!import-software-image-via-url
notes:
  - SDK Method used are
    software_image_management_swim.SoftwareImageManagementSwim.import_software_image_via_url_v1,

  - Paths used are
    post /dna/intent/api/v1/image/importation/source/url,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.swim_import_via_url_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    payload:
    - applicationType: string
      imageFamily: string
      sourceURL: string
      thirdParty: true
      vendor: string
    scheduleAt: string
    scheduleDesc: string
    scheduleOrigin: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: iam_access_key_info
version_added: 2.1.0
version_added_collection: community.aws
short_description: fetch information about AWS IAM User access keys
description:
  - Fetches information AWS IAM user access keys.
  - 'Note: It is not possible to fetch the secret access key.'
author:
  - Mark Chappell (@tremble)
options:
  user_name:
    description:
      - The name of the IAM User to which the keys belong.
    required: true
    type: str
    aliases: ['username']

extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Fetch Access keys for a user
  amazon.aws.iam_access_key_info:
    user_name: example_user
"""

RETURN = r"""
access_key:
    description: A dictionary containing all the access key information.
    returned: When the key exists.
    type: list
    elements: dict
    contains:
        access_key_id:
            description: The ID for the access key.
            returned: success
            type: str
            sample: AKIA1EXAMPLE1EXAMPLE
        create_date:
            description: The date and time, in ISO 8601 date-time format, when the access key was created.
            returned: success
            type: str
            sample: "2021-10-09T13:25:42+00:00"
        user_name:
            description: The name of the IAM user to which the key is attached.
            returned: success
            type: str
            sample: example_user
        status:
            description:
              - The status of the key.
              - C(Active) means it can be used.
              - C(Inactive) means it can not be used.
            returned: success
            type: str
            sample: Inactive
"""

from ansible_collections.amazon.aws.plugins.module_utils.iam import AnsibleIAMError
from ansible_collections.amazon.aws.plugins.module_utils.iam import get_iam_access_keys
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry


def main():
    argument_spec = dict(
        user_name=dict(required=True, type="str", aliases=["username"]),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)

    client = module.client("iam", retry_decorator=AWSRetry.jittered_backoff())

    try:
        access_keys = get_iam_access_keys(client, module.params.get("user_name"))
        module.exit_json(changed=False, access_keys=access_keys)
    except AnsibleIAMError as e:
        module.fail_json_aws_error(e)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for eos_prefix_lists
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
---
module: eos_prefix_lists
short_description: Manages Prefix lists resource module
description: This module configures and manages the attributes of Prefix lists on Arista
  EOS platforms.
version_added: 2.2.0
author: Gomathi Selvi Srinivasan (@GomathiselviS)
notes:
- Tested against Arista EOS 4.24.6F
- This module works with connection C(network_cli). See the L(EOS Platform Options,eos_platform_options).
options:
   config:
      description: A list of dictionary of prefix-list options
      type: list
      elements: dict
      suboptions:
        afi:
          description:
          - The Address Family Indicator (AFI) for the  prefix list.
          type: str
          required: true
          choices:
          - ipv4
          - ipv6
        prefix_lists:
          description:
          - A list of prefix-lists.
          type: list
          elements: dict
          suboptions:
            name:
              description: Name of the prefix-list
              type: str
              required: true
            entries:
              description: List of prefix-lists
              type: list
              elements: dict
              suboptions:
                action:
                  description: action to be performed on the specified path
                  type: str
                  choices: ['deny', 'permit']
                address:
                  description: ipv4/v6 address in prefix-mask or address-masklen format
                  type: str
                match:
                  description: match masklen
                  type: dict
                  suboptions:
                    operator:
                      description: equalto/greater than/lesser than
                      type: str
                      choices: ['eq', 'le', 'ge']
                    masklen:
                      description: Mask Length.
                      type: int
                sequence:
                  description: sequence number
                  type: int
                resequence:
                  description: Resequence the list.
                  type: dict
                  suboptions:
                    default:
                      description: Resequence with default values (10).
                      type: bool
                    start_seq:
                      description: Starting sequence number.
                      type: int
                    step:
                      description: Step to increment the sequence number.
                      type: int
   running_config:
      description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the EOS device by
        executing the command B(show running-config | section access-list).
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into Ansible structured data as per the resource module's argspec
        and the value is then returned in the I(parsed) key within the result.
      type: str
   state:
      description:
      - The state the configuration should be left in.
      type: str
      choices:
      - deleted
      - merged
      - overridden
      - replaced
      - gathered
      - rendered
      - parsed
      default: merged
"""
EXAMPLES = """
# Using merged


# Before state
# veos#show running-config | section prefix-lists
# veos#

- name: Merge provided configuration with device configuration
  arista.eos.eos_prefix_lists:
    config:
      - afi: "ipv4"
        prefix_lists:
          - name: "v401"
            entries:
              - sequence: 25
                action: "deny"
                address: "45.55.4.0/24"
              - sequence: 100
                action: "permit"
                address: "11.11.2.0/24"
                match:
                  masklen: 32
                  operator: "ge"
          - name: "v402"
            entries:
              - action: "deny"
                address: "10.1.1.0/24"
                sequence: 10
                match:
                  masklen: 32
                  operator: "ge"
      - afi: "ipv6"
        prefix_lists:
          - name: "v601"
            entries:
              - sequence: 125
                action: "deny"
                address: "5000:1::/64"

# Task Output
# -------------
# before: {}
# commands:
# - ipv6 prefix-list v601
# - seq 125 deny 5000:1::/64
# - ip prefix-list v401
# - seq 25 deny 45.55.4.0/24
# - seq 100 permit 11.11.2.0/24 ge 32
# - ip prefix-list v402
# - seq 10 deny 10.1.1.0/24 ge 32
# after:
# - afi: ipv4
#   prefix_lists:
#   - entries:
#     - action: deny
#       address: 45.55.4.0/24
#       sequence: 25
#     - action: permit
#       address: 11.11.2.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 100
#     name: v401
#   - entries:
#     - action: deny
#       address: 10.1.1.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 10
#     name: v402
# - afi: ipv6
#   prefix_lists:
#   - entries:
#     - action: deny
#       address: 5000:1::/64
#       sequence: 125
#     name: v601


# After state:
# ------------
# veos#
# veos#show running-config | section prefix-list
# ip prefix-list v401
#    seq 25 deny 45.55.4.0/24
#    seq 100 permit 11.11.2.0/24 ge 32
# !
# ip prefix-list v402
#    seq 10 deny 10.1.1.0/24 ge 32
# !
# ipv6 prefix-list v601
#    seq 125 deny 5000:1::/64
# veos#


# Using merged:
# Failure scenario : 'merged' should not be used when an existing prefix-list (sequence number)
# is to be modified.


# veos#show running-config | section prefix-list
# ip prefix-list v401
#    seq 25 deny 45.55.4.0/24
#    seq 100 permit 11.11.2.0/24 ge 32
# !
# ip prefix-list v402
#    seq 10 deny 10.1.1.0/24 ge 32
# !
# ipv6 prefix-list v601
#    seq 125 deny 5000:1::/64
# veos#

- name: Merge provided configuration with device configuration
  arista.eos.eos_prefix_lists:
    config:
      - afi: "ipv4"
        prefix_lists:
          - name: "v401"
            entries:
              - sequence: 25
                action: "deny"
                address: "45.55.4.0/24"
                match:
                  masklen: 32
                  operator: "ge"
              - sequence: 100
                action: "permit"
                address: "11.11.2.0/24"
                match:
                  masklen: 32
                  operator: "ge"
          - name: "v402"
            entries:
              - action: "deny"
                address: "10.1.1.0/24"
                sequence: 10
                match:
                  masklen: 32
                  operator: "ge"
      - afi: "ipv6"
        prefix_lists:
          - name: "v601"
            entries:
              - sequence: 125
                action: "deny"
                address: "5000:1::/64"
    state: merged

# Task Output
# -------------
# changed: false
# invocation:
#   module_args:
#     config:
#     - afi: ipv4
#       prefix_lists:
#       - entries:
#         - action: deny
#           address: 45.55.4.0/24
#           match:
#             masklen: 32
#             operator: ge
#           resequence:
#           sequence: 25
#         - action: permit
#           address: 11.11.2.0/24
#           match:
#             masklen: 32
#             operator: ge
#           resequence:
#           sequence: 100
#         name: v401
#       - entries:
#         - action: deny
#           address: 10.1.1.0/24
#           match:
#             masklen: 32
#             operator: ge
#           resequence:
#           sequence: 10
#         name: v402
#     - afi: ipv6
#       prefix_lists:
#       - entries:
#         - action: deny
#           address: 5000:1::/64
#           match:
#           resequence:
#           sequence: 125
#         name: v601
#     running_config:
#     state: merged
# msg: Sequence number 25 is already present. Use replaced/overridden operation to change
#   the configuration


# Using Replaced:

# Before state:
# veos#show running-config | section prefix-list
# ip prefix-list v401
#    seq 25 deny 45.55.4.0/24
#    seq 100 permit 11.11.2.0/24 ge 32
# !
# ip prefix-list v402
#    seq 10 deny 10.1.1.0/24 ge 32
# !
# ipv6 prefix-list v601
#    seq 125 deny 5000:1::/64
# veos#


- name: Replace Provided configuration with given configuration
  arista.eos.eos_prefix_lists:
    config:
      - afi: "ipv4"
        prefix_lists:
          - name: "v401"
            entries:
              - sequence: 25
                action: "deny"
                address: "45.55.4.0/24"
                match:
                  masklen: 32
                  operator: "ge"
              - sequence: 200
                action: "permit"
                address: "200.11.2.0/24"
                match:
                  masklen: 32
                  operator: "ge"
    state: replaced


# Task Output
# -------------
# before:
# - afi: ipv4
#   prefix_lists:
#   - entries:
#     - action: deny
#       address: 45.55.4.0/24
#       sequence: 25
#     - action: permit
#       address: 11.11.2.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 100
#     name: v401
#   - entries:
#     - action: deny
#       address: 10.1.1.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 10
#     name: v402
# - afi: ipv6
#   prefix_lists:
#   - entries:
#     - action: deny
#       address: 5000:1::/64
#       sequence: 125
#     name: v601
# commands:
# - ip prefix-list v401
# - no seq 25
# - seq 25 deny 45.55.4.0/24 ge 32
# - seq 200 permit 200.11.2.0/24 ge 32
# - no seq 100
# - no ip prefix-list v402
# after:
# - afi: ipv4
#   prefix_lists:
#   - entries:
#     - action: deny
#       address: 45.55.4.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 25
#     - action: permit
#       address: 200.11.2.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 200
#     name: v401
# - afi: ipv6
#   prefix_lists:
#   - entries:
#     - action: deny
#       address: 5000:1::/64
#       sequence: 125
#     name: v601


# After State:
# veos#show running-config | section prefix-list
# ip prefix-list v401
#    seq 25 deny 45.55.4.0/24 ge 32
#    seq 200 permit 200.11.2.0/24 ge 32
# !
# ipv6 prefix-list v601
#    seq 125 deny 5000:1::/64
# veos#
#
#


# Using overridden:


# Before State:
# veos#show running-config | section prefix-list
# ip prefix-list v401
#    seq 25 deny 45.55.4.0/24 ge 32
#    seq 100 permit 11.11.2.0/24 ge 32
#    seq 200 permit 200.11.2.0/24 ge 32
# !
# ip prefix-list v402
#    seq 10 deny 10.1.1.0/24 ge 32
# !
# ipv6 prefix-list v601
#    seq 125 deny 5000:1::/64
# veos#

- name: Override
  arista.eos.eos_prefix_lists:
    config:
      - afi: "ipv4"
        prefix_lists:
          - name: "v401"
            entries:
              - sequence: 25
                action: "deny"
                address: "45.55.4.0/24"
              - sequence: 300
                action: "permit"
                address: "30.11.2.0/24"
                match:
                  masklen: 32
                  operator: "ge"
          - name: "v403"
            entries:
              - action: "deny"
                address: "10.1.1.0/24"
                sequence: 10
    state: overridden


# Task Output
# -------------
# before:
# - afi: ipv4
#   prefix_lists:
#   - entries:
#     - action: deny
#       address: 45.55.4.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 25
#     - action: permit
#       address: 11.11.2.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 100
#     - action: permit
#       address: 200.11.2.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 200
#     name: v401
#   - entries:
#     - action: deny
#       address: 10.1.1.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 10
#     name: v402
# - afi: ipv6
#   prefix_lists:
#   - entries:
#     - action: deny
#       address: 5000:1::/64
#       sequence: 125
#     name: v601
# commands:
# - no ipv6 prefix-list v601
# - ip prefix-list v401
# - seq 25 deny 45.55.4.0/24
# - seq 300 permit 30.11.2.0/24 ge 32
# - no seq 100
# - no seq 200
# - ip prefix-list v403
# - seq 10 deny 10.1.1.0/24
# - no ip prefix-list v402
# after:
# - afi: ipv4
#   prefix_lists:
#   - entries:
#     - action: deny
#       address: 45.55.4.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 25
#     - action: permit
#       address: 30.11.2.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 300
#     name: v401
#   - entries:
#     - action: deny
#       address: 10.1.1.0/24
#       sequence: 10
#     name: v403


# After State
# veos#
# veos#show running-config | section prefix-list
# ip prefix-list v401
#    seq 25 deny 45.55.4.0/24 ge 32
#    seq 300 permit 30.11.2.0/24 ge 32
# !
# ip prefix-list v403
#    seq 10 deny 10.1.1.0/24
# veos#

# Using deleted:

# Before State:
# veos#show running-config | section prefix-list
# ip prefix-list v401
#    seq 25 deny 45.55.4.0/24 ge 32
#    seq 100 permit 11.11.2.0/24 ge 32
#    seq 300 permit 30.11.2.0/24 ge 32
# !
# ip prefix-list v402
#    seq 10 deny 10.1.1.0/24 ge 32
# !
# ip prefix-list v403
#    seq 10 deny 10.1.1.0/24
# !
# ipv6 prefix-list v601
#    seq 125 deny 5000:1::/64
# veos#

- name: Delete device configuration
  arista.eos.eos_prefix_lists:
    config:
      - afi: "ipv6"
    state: deleted

# Task Output
# -------------
# before:
# - afi: ipv4
#   prefix_lists:
#   - entries:
#     - action: deny
#       address: 45.55.4.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 25
#     - action: permit
#       address: 11.11.2.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 100
#     - action: permit
#       address: 30.11.2.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 300
#     name: v401
#   - entries:
#     - action: deny
#       address: 10.1.1.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 10
#     name: v402
#   - entries:
#     - action: deny
#       address: 10.1.1.0/24
#       sequence: 10
#     name: v403
# - afi: ipv6
#   prefix_lists:
#   - entries:
#     - action: deny
#       address: 5000:1::/64
#       sequence: 125
#     name: v601
# commands:
# - no ipv6 prefix-list v601
# after:
# - afi: ipv4
#   prefix_lists:
#   - entries:
#     - action: deny
#       address: 45.55.4.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 25
#     - action: permit
#       address: 11.11.2.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 100
#     - action: permit
#       address: 30.11.2.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 300
#     name: v401
#   - entries:
#     - action: deny
#       address: 10.1.1.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 10
#     name: v402
#   - entries:
#     - action: deny
#       address: 10.1.1.0/24
#       sequence: 10
#     name: v403

# after State:
# veos#show running-config | section prefix-list
# ip prefix-list v401
#    seq 25 deny 45.55.4.0/24 ge 32
#    seq 100 permit 11.11.2.0/24 ge 32
#    seq 300 permit 30.11.2.0/24 ge 32
# !
# ip prefix-list v402
#    seq 10 deny 10.1.1.0/24 ge 32
# !
# ip prefix-list v403
#    seq 10 deny 10.1.1.0/24
#


# Using deleted


# Before state:
# veos#show running-config | section prefix-list
# ip prefix-list v401
#    seq 25 deny 45.55.4.0/24 ge 32
#    seq 100 permit 11.11.2.0/24 ge 32
#    seq 300 permit 30.11.2.0/24 ge 32
# !
# ip prefix-list v402
#    seq 10 deny 10.1.1.0/24 ge 32
# !
# ip prefix-list v403
#    seq 10 deny 10.1.1.0/24
# veos#

- name: Delete device configuration
  arista.eos.eos_prefix_lists:
    state: deleted


# Task Output
# -------------
# before:
# - afi: ipv4
#   prefix_lists:
#   - entries:
#     - action: deny
#       address: 45.55.4.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 25
#     - action: permit
#       address: 11.11.2.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 100
#     - action: permit
#       address: 30.11.2.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 300
#     name: v401
#   - entries:
#     - action: deny
#       address: 10.1.1.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 10
#     name: v402
#   - entries:
#     - action: deny
#       address: 10.1.1.0/24
#       sequence: 10
#     name: v403
# commands:
# - no ip prefix-list v401
# - no ip prefix-list v402
# - no ip prefix-list v403
# after: {}

# After State:
# veos#show running-config | section prefix-list
# veos#


# Using parsed:


# parse_prefix_lists.cfg
# ip prefix-list v401
#    seq 25 deny 45.55.4.0/24
#    seq 100 permit 11.11.2.0/24 ge 32
# !
# ip prefix-list v402
#    seq 10 deny 10.1.1.0/24
# !
# ipv6 prefix-list v601
#    seq 125 deny 5000:1::/64
#


- name: parse configs
  arista.eos.eos_prefix_lists:
    running_config: "{{ lookup('file', './parsed_prefix_lists.cfg') }}"
    state: parsed


# Task Output
# -------------
# parsed:
# - afi: ipv4
#   prefix_lists:
#   - entries:
#     - action: deny
#       address: 45.55.4.0/24
#       sequence: 25
#     - action: permit
#       address: 11.11.2.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 100
#     name: v401
#   - entries:
#     - action: deny
#       address: 10.1.1.0/24
#       sequence: 10
#     name: v402
# - afi: ipv6
#   prefix_lists:
#   - entries:
#     - action: deny
#       address: 5000:1::/64
#       sequence: 125
#     name: v601


# Using rendered:

- name: Render provided configuration
  arista.eos.eos_prefix_lists:
    config:
      - afi: "ipv4"
        prefix_lists:
          - name: "v401"
            entries:
              - sequence: 25
                action: "deny"
                address: "45.55.4.0/24"
              - sequence: 200
                action: "permit"
                address: "200.11.2.0/24"
                match:
                  masklen: 32
                  operator: "ge"
          - name: "v403"
            entries:
              - action: "deny"
                address: "10.1.1.0/24"
                sequence: 10
    state: rendered

# Task Output
# -------------
# rendered:
# - ip prefix-list v401
# - seq 25 deny 45.55.4.0/24
# - seq 200 permit 200.11.2.0/24 ge 32
# - ip prefix-list v403
# - seq 10 deny 10.1.1.0/24

# using gathered:


# Device config:
# veos#show running-config | section prefix-list
# ip prefix-list v401
#    seq 25 deny 45.55.4.0/24
#    seq 100 permit 11.11.2.0/24 ge 32
# !
# ip prefix-list v402
#    seq 10 deny 10.1.1.0/24 ge 32
# !
# ipv6 prefix-list v601
#    seq 125 deny 5000:1::/64
# veos#

- name: gather configs
  arista.eos.eos_prefix_lists:
    state: gathered

# Task Output
# -------------
# gathered:
# - afi: ipv4
#   prefix_lists:
#   - entries:
#     - action: deny
#       address: 45.55.4.0/24
#       sequence: 25
#     - action: permit
#       address: 11.11.2.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 100
#     name: v401
#   - entries:
#     - action: deny
#       address: 10.1.1.0/24
#       match:
#         masklen: 32
#         operator: ge
#       sequence: 10
#     name: v402
# - afi: ipv6
#   prefix_lists:
#   - entries:
#     - action: deny
#       address: 5000:1::/64
#       sequence: 125
#     name: v601
"""
RETURN = """
before:
  description: The configuration prior to the model invocation.
  returned: always
  type: list
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
after:
  description: The resulting configuration model invocation.
  returned: when changed
  type: list
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
commands:
  description: The set of commands pushed to the remote device.
  returned: always
  type: list
  sample:
    - ip prefix-list v401
    - seq 25 deny 45.55.4.0/24
    - seq 200 permit 200.11.2.0/24 ge 32
    - ip prefix-list v403
    - seq 10 deny 10.1.1.0/24
rendered:
  description: The set of CLI commands generated from the value in C(config) option
  returned: When C(state) is I(rendered)
  type: list
  sample: >
    - ip prefix-list v401
    - seq 25 deny 45.55.4.0/24
    - seq 200 permit 200.11.2.0/24 ge 32
    - ip prefix-list v403
    - seq 10 deny 10.1.1.0/24
gathered:
  description: The configuration as structured data transformed for the running configuration
               fetched from remote host
  returned: When C(state) is I(gathered)
  type: list
  sample: >
    The configuration returned will always be in the same format
    of the parameters above.
parsed:
  description: The configuration as structured data transformed for the value of
               C(running_config) option
  returned: When C(state) is I(parsed)
  type: list
  sample: >
    The configuration returned will always be in the same format
    of the parameters above.
"""


from ansible.module_utils.basic import AnsibleModule

from ansible_collections.arista.eos.plugins.module_utils.network.eos.argspec.prefix_lists.prefix_lists import (
    Prefix_listsArgs,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.config.prefix_lists.prefix_lists import (
    Prefix_lists,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Prefix_listsArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Prefix_lists(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Christian Kolrep <christian.kolrep@dataport.de>
# Copyright: (c) 2024, Akini Ross <akinross@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_epg_useg_attribute_simple_statement
short_description: Manage EPG useg Attributes Simple Statements (fv:DnsAttr, fv:IdGroupAttr, fv:IpAttr, fv:MacAttr, and fv:VmAttr)
description:
- Manage EPG useg Attributes Simple Statements
options:
  tenant:
    description:
    - The name of an existing tenant.
    type: str
    aliases: [ tenant_name ]
  ap:
    description:
    - The name of an existing application network profile.
    type: str
    aliases: [ app_profile, app_profile_name ]
  epg:
    description:
    - The name of an existing end point group.
    type: str
    aliases: [ epg_name ]
  parent_block_statements:
    description:
    - The list of parent block statements.
    - The order of the provided list matters, assuming the list ["A", "B", "C"].
    - The block statement "A" will be the parent of "B"
    - The block statement "A" will be a child of the default block statement.
    - The maximum amount of parent block statements is 3.
    type: list
    elements: str
    aliases: [ blocks, parent_blocks ]
  name:
    description:
    - The name of the EPG useg attribute.
    type: str
    aliases: [ useg_attribute_name ]
  type:
    description:
    - The type of the EPG useg attribute
    type: str
    required: true
    choices:
    - ip
    - mac
    - dns
    - ad_group
    - vm_custom_attr
    - vm_vmm_domain
    - vm_operating_system
    - vm_hypervisor_id
    - vm_datacenter
    - vm_id
    - vm_name
    - vm_folder
    - vm_folder_path
    - vm_vnic
    - vm_tag
    aliases: [ useg_attribute_type ]
  operator:
    description:
    - The operator of the EPG useg attribute.
    type: str
    choices: [ equals, contains, starts_with, ends_with ]
  category:
    description:
    - The name of the vmware tag category or vmware custom attribute.
    type: str
    aliases: [ custom_attribute ]
  use_subnet:
    description:
    - Whether to use the EPG subnet definition for ip.
    type: bool
  value:
    description:
    - The value of the EPG useg attribute.
    type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

notes:
- The I(tenant), I(ap) and I(epg) used must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant), M(cisco.aci.aci_ap) and M(cisco.aci.aci_epg) modules can be used for this.
seealso:
- module: cisco.aci.aci_tenant
- module: cisco.aci.aci_ap
- module: cisco.aci.aci_epg
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(fv:DnsAttr), B(fv:IdGroupAttr), B(fv:IpAttr), B(fv:MacAttr), and B(fv:VmAttr).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Christian Kolrep (@Christian-Kolrep)
- Akini Ross (@akinross)
"""

EXAMPLES = r"""
- name: Add a new vmtag useg attribute in default block statement
  cisco.aci.aci_epg_useg_attribute_simple_statement:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: anstest
    ap: anstest
    epg: anstest
    name: vmtagprod
    type: vmtag
    category: Environment
    operator: equals
    value: Production
    state: present
  delegate_to: localhost

- name: Add a new vmtag useg attribute in nested block statement
  cisco.aci.aci_epg_useg_attribute_simple_statement:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: anstest
    ap: anstest
    epg: anstest
    name: vmtagprod
    parent_block_statements:
      - block_a
      - block_b
    type: vmtag
    category: Environment
    operator: equals
    value: Production
    state: present
  delegate_to: localhost

- name: Query a specific vmtag useg attribute in default block statement
  cisco.aci.aci_epg_useg_attribute_simple_statement:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: anstest
    ap: anstest
    epg: anstest
    name: vmtagprod
    type: vmtag
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all vmtag useg attributes
  cisco.aci.aci_epg_useg_attribute_simple_statement:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
    type: vmtag
  delegate_to: localhost
  register: query_result

- name: Remove an existing vmtag useg attribute from default block statement
  cisco.aci.aci_epg_useg_attribute_simple_statement:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: anstest
    ap: anstest
    epg: anstest
    name: vmtagprod
    type: vmtag
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec
from ansible_collections.cisco.aci.plugins.module_utils.constants import USEG_ATTRIBUTE_MAPPING, OPERATOR_MAPPING


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        ap=dict(type="str", aliases=["app_profile", "app_profile_name"]),  # Not required for querying all objects
        epg=dict(type="str", aliases=["epg_name"]),  # Not required for querying all objects
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        tenant=dict(type="str", aliases=["tenant_name"]),  # Not required for querying all objects
        parent_block_statements=dict(type="list", elements="str", aliases=["parent_blocks", "blocks"]),
        name=dict(type="str", aliases=["useg_attribute_name"]),
        type=dict(type="str", required=True, choices=list(USEG_ATTRIBUTE_MAPPING.keys()), aliases=["useg_attribute_type"]),
        operator=dict(type="str", choices=list(OPERATOR_MAPPING.keys())),
        category=dict(type="str", aliases=["custom_attribute"]),
        value=dict(type="str"),
        use_subnet=dict(type="bool"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["ap", "epg", "tenant", "name"]],
            ["state", "present", ["ap", "epg", "tenant", "name"]],
        ],
    )

    aci = ACIModule(module)

    ap = module.params.get("ap")
    epg = module.params.get("epg")
    state = module.params.get("state")
    tenant = module.params.get("tenant")
    blocks = module.params.get("parent_block_statements")
    name = module.params.get("name")
    attribute_type = module.params.get("type")
    value = module.params.get("value")
    operator = module.params.get("operator")
    category = module.params.get("category")
    use_subnet = aci.boolean(module.params.get("use_subnet"))

    # Excluding below classes from the module:
    # fvProtoAttr:
    #   Was used in AVS, but it is not longer in use.
    # fvUsegBDCont:
    #   Was part of a feature that allowed uSeg attributes to be applied at VRF (instead of BD) level.
    #   It has been since deprecated and we no longer allow setting the scope at fvCtrn to scope-vrf.
    #   This type of functionality has been replaced by the ESG feature.
    attribute_class = USEG_ATTRIBUTE_MAPPING[attribute_type]["attribute_class"]
    attribute_rn = USEG_ATTRIBUTE_MAPPING[attribute_type]["rn_format"].format(name)
    attribute_type = USEG_ATTRIBUTE_MAPPING[attribute_type]["attribute_type"]

    if blocks:
        if len(blocks) > 3:
            module.fail_json(msg="{0} block statements are provided but the maximum amount of parent_block_statements is 3".format(len(blocks)))
        parent_blocks_class = "fvSCrtrn"
        parent_blocks_rn = "crtrn/crtrn-{0}".format("/crtrn-".join(blocks))
        parent_blocks_name = blocks[-1]
    else:
        parent_blocks_class = "fvCrtrn"
        parent_blocks_rn = "crtrn"
        parent_blocks_name = "default"

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="fvAp",
            aci_rn="ap-{0}".format(ap),
            module_object=ap,
            target_filter={"name": ap},
        ),
        subclass_2=dict(
            aci_class="fvAEPg",
            aci_rn="epg-{0}".format(epg),
            module_object=epg,
            target_filter={"name": epg},
        ),
        subclass_3=dict(
            aci_class=parent_blocks_class,
            aci_rn=parent_blocks_rn,
            module_object=parent_blocks_name,
            target_filter={"name": parent_blocks_name},
        ),
        subclass_4=dict(
            aci_class=attribute_class,
            aci_rn=attribute_rn,
            module_object=name,
            target_filter={"name": name},
        ),
    )

    aci.get_existing()

    if state == "present":
        class_config = dict(name=name)

        if attribute_class == "fvVmAttr":
            class_config.update(type=attribute_type)
            class_config.update(operator=OPERATOR_MAPPING.get(operator))
            class_config.update(value=value)
            if attribute_type == "tag":
                class_config.update(category=category)
            elif attribute_type == "custom-label":
                class_config.update(labelName=category)

        elif attribute_class == "fvIpAttr":
            class_config.update(usefvSubnet=use_subnet)
            class_config.update(ip=value)

        elif attribute_class == "fvMacAttr":
            class_config.update(mac=value.upper())

        elif attribute_class == "fvDnsAttr":
            class_config.update(filter=value)

        elif attribute_class == "fvIdGroupAttr":
            class_config.update(selector=value)

        aci.payload(aci_class=attribute_class, class_config=class_config)

        aci.get_diff(aci_class=attribute_class)

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2019 Yuwei Zhou, <yuwzho@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_lock_info
version_added: "0.1.2"
short_description: Manage Azure locks
description:
    - Create, delete an Azure lock.
options:
    name:
        description:
            - Name of the lock.
        type: str
    managed_resource_id:
        description:
            - ID of the resource where need to manage the lock.
            - Get this via facts module.
            - Cannot be set mutual with I(resource_group).
            - Manage subscription if both I(managed_resource_id) and I(resource_group) not defined.
            - "'/subscriptions/{subscriptionId}' for subscriptions."
            - "'/subscriptions/{subscriptionId}/resourcegroups/{resourceGroupName}' for resource groups."
            - "'/subscriptions/{subscriptionId}/resourcegroups/{resourceGroupName}/providers/{namespace}/{resourceType}/{resourceName}' for resources."
            - Can get all locks with 'child scope' for this resource, use I(managed_resource_id) in response for further management.
        type: str
    resource_group:
        description:
            - Resource group name where need to manage the lock.
            - The lock is in the resource group level.
            - Cannot be set mutual with I(managed_resource_id).
            - Query subscription if both I(managed_resource_id) and I(resource_group) not defined.
            - Can get all locks with 'child scope' in this resource group, use the I(managed_resource_id) in response for further management.
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Yuwei Zhou (@yuwzho)

'''

EXAMPLES = '''
- name: Get myLock details of myVM
  azure_rm_lock_info:
    name: myLock
    managed_resource_id: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourcegroups/myResourceGroup/providers/Microsoft.Compute/virtualMachines/myVM

- name: List locks of myVM
  azure_rm_lock_info:
    managed_resource_id: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourcegroups/myResourceGroup/providers/Microsoft.Compute/virtualMachines/myVM

- name: List locks of myResourceGroup
  azure_rm_lock_info:
    resource_group: myResourceGroup

- name: List locks of myResourceGroup
  azure_rm_lock_info:
    managed_resource_id: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourcegroups/myResourceGroup

- name: List locks of mySubscription
  azure_rm_lock_info:

- name: List locks of mySubscription
  azure_rm_lock_info:
    managed_resource_id: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
'''

RETURN = '''
locks:
    description:
        - List of locks dicts.
    returned: always
    type: complex
    contains:
        id:
            description:
                - ID of the Lock.
            returned: always
            type: str
            sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Authorization/locks/myLock"
        name:
            description:
                - Name of the lock.
            returned: always
            type: str
            sample: myLock
        level:
            description:
                - Type level of the lock.
            returned: always
            type: str
            sample: can_not_delete
        notes:
            description:
                - Notes of the lock added by creator.
            returned: always
            type: str
            sample: "This is a lock"
'''  # NOQA

import json
import re
from ansible.module_utils.common.dict_transformations import _camel_to_snake
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_rest import GenericRestClient

try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMLockInfo(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(type='str'),
            resource_group=dict(type='str'),
            managed_resource_id=dict(type='str')
        )

        self.results = dict(
            changed=False,
            locks=[]
        )

        mutually_exclusive = [['resource_group', 'managed_resource_id']]

        self.name = None
        self.resource_group = None
        self.managed_resource_id = None
        self._mgmt_client = None
        self._query_parameters = {'api-version': '2016-09-01'}
        self._header_parameters = {'Content-Type': 'application/json; charset=utf-8'}

        super(AzureRMLockInfo, self).__init__(self.module_arg_spec,
                                              supports_check_mode=True,
                                              facts_module=True,
                                              mutually_exclusive=mutually_exclusive,
                                              supports_tags=False)

    def exec_module(self, **kwargs):

        is_old_facts = self.module._name == 'azure_rm_lock_facts'
        if is_old_facts:
            self.module.deprecate("The 'azure_rm_lock_facts' module has been renamed to 'azure_rm_lock_info'", version=(2.9, ))

        for key in self.module_arg_spec.keys():
            setattr(self, key, kwargs[key])

        self._mgmt_client = self.get_mgmt_svc_client(GenericRestClient, base_url=self._cloud_environment.endpoints.resource_manager)
        changed = False
        # construct scope id
        scope = self.get_scope()
        url = '/{0}/providers/Microsoft.Authorization/locks'.format(scope)
        if self.name:
            url = '{0}/{1}'.format(url, self.name)
        locks = self.list_locks(url)
        resp = locks.get('value') if 'value' in locks else [locks]
        self.results['locks'] = [self.to_dict(x) for x in resp]
        return self.results

    def to_dict(self, lock):
        resp = dict(
            id=lock['id'],
            name=lock['name'],
            level=_camel_to_snake(lock['properties']['level']),
            managed_resource_id=re.sub('/providers/Microsoft.Authorization/locks/.+', '', lock['id'])
        )
        if lock['properties'].get('notes'):
            resp['notes'] = lock['properties']['notes']
        if lock['properties'].get('owners'):
            resp['owners'] = [x['application_id'] for x in lock['properties']['owners']]
        return resp

    def list_locks(self, url):
        try:
            resp = self._mgmt_client.query(url=url,
                                           method='GET',
                                           query_parameters=self._query_parameters,
                                           header_parameters=self._header_parameters,
                                           body=None,
                                           expected_status_codes=[200],
                                           polling_timeout=None,
                                           polling_interval=None)
            return json.loads(resp.body())
        except Exception as exc:
            self.fail('Error when finding locks {0}: {1}'.format(url, exc.message))

    def get_scope(self):
        '''
        Get the resource scope of the lock management.
        '/subscriptions/{subscriptionId}' for subscriptions,
        '/subscriptions/{subscriptionId}/resourcegroups/{resourceGroupName}' for resource groups,
        '/subscriptions/{subscriptionId}/resourcegroups/{resourceGroupName}/providers/{namespace}/{resourceType}/{resourceName}' for resources.
        '''
        if self.managed_resource_id:
            return self.managed_resource_id
        elif self.resource_group:
            return '/subscriptions/{0}/resourcegroups/{1}'.format(self.subscription_id, self.resource_group)
        else:
            return '/subscriptions/{0}'.format(self.subscription_id)


def main():
    AzureRMLockInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2014-2017 Ansible Project
# Copyright (c) 2017, 2018 Will Thames
# Copyright (c) 2017, 2018 Michael De La Rue
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: rds_snapshot_info
version_added: 5.0.0
short_description: obtain information about one or more RDS snapshots
description:
  - Obtain information about one or more RDS snapshots. These can be for unclustered snapshots or snapshots of clustered DBs (Aurora).
  - Aurora snapshot information may be obtained if no identifier parameters are passed or if one of the cluster parameters are passed.
  - This module was originally added to C(community.aws) in release 1.0.0.
options:
  db_snapshot_identifier:
    description:
      - Name of an RDS (unclustered) snapshot.
      - Mutually exclusive with O(db_instance_identifier), O(db_cluster_identifier), O(db_cluster_snapshot_identifier).
    required: false
    aliases:
      - snapshot_name
    type: str
  db_instance_identifier:
    description:
      - RDS instance name for which to find snapshots.
      - Mutually exclusive with O(db_snapshot_identifier), O(db_cluster_identifier), O(db_cluster_snapshot_identifier).
    required: false
    type: str
  db_cluster_identifier:
    description:
      - RDS cluster name for which to find snapshots.
      - Mutually exclusive with O(db_snapshot_identifier), O(db_instance_identifier), O(db_cluster_snapshot_identifier).
    required: false
    type: str
  db_cluster_snapshot_identifier:
    description:
      - Name of an RDS cluster snapshot.
      - Mutually exclusive with O(db_instance_identifier), O(db_snapshot_identifier), O(db_cluster_identifier).
    required: false
    type: str
  snapshot_type:
    description:
      - Type of snapshot to find.
      - By default both automated and manual snapshots will be returned.
    required: false
    choices: ['automated', 'manual', 'shared', 'public']
    type: str
author:
  - "Will Thames (@willthames)"
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Get information about an snapshot
  amazon.aws.rds_snapshot_info:
    db_snapshot_identifier: snapshot_name
  register: new_database_info

- name: Get all RDS snapshots for an RDS instance
  amazon.aws.rds_snapshot_info:
    db_instance_identifier: helloworld-rds-master
"""

RETURN = r"""
snapshots:
  description: List of non-clustered snapshots.
  returned: When cluster parameters are not passed.
  type: complex
  contains:
    allocated_storage:
      description: The allocated storage size in gibibytes (GiB).
      returned: always
      type: int
      sample: 10
    availability_zone:
      description: The name of the Availability Zone the DB instance was located in at the time of the DB snapshot.
      returned: always
      type: str
      sample: "us-west-2b"
    db_instance_identifier:
      description: The DB instance identifier of the DB instance this DB snapshot was created from.
      returned: always
      type: str
      sample: "hello-world-rds"
    db_snapshot_arn:
      description: The Amazon Resource Name (ARN) for the DB snapshot.
      returned: always
      type: str
      sample: "arn:aws:rds:us-west-2:123456789012:snapshot:rds:hello-world-rds-us1-2018-05-16-04-03"
    db_snapshot_identifier:
      description: The identifier for the DB snapshot.
      returned: always
      type: str
      sample: "rds:hello-world-rds-us1-2018-05-16-04-03"
    dbi_resource_id:
      description: The identifier for the source DB instance, which can't be changed and which is unique to an Amazon Web Services Region.
      returned: always
      type: str
      sample: "db-ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    dedicated_log_volume:
      description: Whether the DB instance has a dedicated log volume (DLV) enabled.
      returned: always
      type: bool
      sample: false
    encrypted:
      description: Whether the DB snapshot is encrypted.
      returned: always
      type: bool
      sample: true
    engine:
      description: The name of the database engine.
      returned: always
      type: str
      sample: "postgres"
    engine_version:
      description: The version of the database engine.
      returned: always
      type: str
      sample: "9.5.10"
    iam_database_authentication_enabled:
      description: Whether mapping of Amazon Web Services Identity and Access Management (IAM) accounts to database accounts is enabled.
      returned: always
      type: bool
      sample: false
    instance_create_time:
      description: The time when the DB instance, from which the snapshot was taken, was created.
      returned: always
      type: str
      sample: "2017-10-10T04:00:07.434000+00:00"
    iops:
      description: The Provisioned IOPS (I/O operations per second) value of the DB instance at the time of the snapshot.
      returned: always
      type: int
      sample: 3000
    kms_key_id:
      description: The Amazon Web Services KMS key identifier for the encrypted DB snapshot.
      returned: when encrypted is true
      type: str
      sample: "arn:aws:kms:us-west-2:123456789012:key/abcd1234-1234-aaaa-0000-1234567890ab"
    license_model:
      description: License model information for the restored DB instance.
      returned: always
      type: str
      sample: "postgresql-license"
    master_username:
      description: The master username for the DB snapshot.
      returned: always
      type: str
      sample: "dbadmin"
    multi_tenant:
      description: Whether the snapshot is of a DB instance using the multi-tenant configuration (TRUE) or the single-tenant configuration (FALSE).
      returned: always
      type: bool
      sample: false
    option_group_name:
      description: The option group name for the DB snapshot.
      returned: always
      type: str
      sample: "default:postgres-9-5"
    original_snapshot_create_time:
      description: The time of the CreateDBSnapshot operation. Doesn't change when the snapshot is copied.
      returned: always
      type: str
      sample: "2017-10-10T04:00:07.434000+00:00"
    percent_progress:
      description: The percentage of the estimated data that has been transferred.
      returned: always
      type: int
      sample: 100
    port:
      description: The port that the database engine was listening on at the time of the snapshot.
      returned: always
      type: int
      sample: 0
    snapshot_create_time:
      description: When the snapshot was taken.
      returned: always
      type: str
      sample: "2018-05-16T04:03:33.871000+00:00"
    snapshot_target:
      description: "Where manual snapshots are stored: Amazon Web Services Outposts or the Amazon Web Services Region."
      returned: always
      type: str
      sample: "region"
    snapshot_type:
      description: The type of the DB snapshot.
      returned: always
      type: str
      sample: "automated"
    status:
      description: The status of this DB snapshot.
      returned: always
      type: str
      sample: "available"
    storage_throughput:
      description: The storage throughput for the DB snapshot.
      returned: always
      type: int
      sample: 500
    storage_type:
      description: The storage type associated with DB snapshot.
      returned: always
      type: str
      sample: "gp2"
    tags:
      description: Tags of the snapshot.
      returned: when snapshot is not shared.
      type: complex
      contains: {}
    vpc_id:
      description: The VPC ID associated with the DB snapshot.
      returned: always
      type: str
      sample: "vpc-abcd1234"
cluster_snapshots:
  description: List of cluster snapshots.
  returned: When cluster parameters are passed.
  type: complex
  contains:
    allocated_storage:
      description: The allocated storage size in gibibytes (GiB).
      returned: always
      type: int
      sample: 1
    availability_zones:
      description: The list of Availability Zones (AZs) where instances in the DB cluster snapshot can be restored.
      returned: always
      type: list
      sample:
      - "ca-central-1a"
      - "ca-central-1b"
    cluster_create_time:
      description: The time when the DB cluster was created.
      returned: always
      type: str
      sample: "2018-05-17T00:13:40.223000+00:00"
    db_cluster_identifier:
      description: The DB cluster identifier of the DB cluster that this DB cluster snapshot was created from.
      returned: always
      type: str
      sample: "test-aurora-cluster"
    db_cluster_resource_id:
      description: The resource ID of the DB cluster that this DB cluster snapshot was created from.
      returned: always
      type: str
      sample: "cluster-ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    db_cluster_snapshot_arn:
      description: The Amazon Resource Name (ARN) for the DB cluster snapshot.6
      returned: always
      type: str
      sample: "arn:aws:rds:ca-central-1:123456789012:cluster-snapshot:test-aurora-snapshot"
    db_cluster_snapshot_identifier:
      description: The identifier for the DB cluster snapshot.
      returned: always
      type: str
      sample: "test-aurora-snapshot"
    engine:
      description: The name of the database engine for this DB cluster snapshot.
      returned: always
      type: str
      sample: "aurora"
    engine_mode:
      description: The engine mode of the database engine for this DB cluster snapshot.
      returned: always
      type: str
      sample: "provisioned"
    engine_version:
      description: The version of the database engine for this DB cluster snapshot.
      returned: always
      type: str
      sample: "5.6.10a"
    iam_database_authentication_enabled:
      description: Whether mapping of Amazon Web Services Identity and Access Management (IAM) accounts to database accounts is enabled.
      returned: always
      type: bool
      sample: false
    kms_key_id:
      description: the Amazon Web Services KMS key identifier for the encrypted DB cluster snapshot.
      returned: when storage_encrypted is true
      type: str
      sample: "arn:aws:kms:us-west-2:123456789012:key/abcd1234-1234-aaaa-0000-1234567890ab"
    license_model:
      description: The license model information for this DB cluster snapshot.
      returned: always
      type: str
      sample: "aurora"
    master_username:
      description: The master username for this DB cluster snapshot.
      returned: always
      type: str
      sample: "shertel"
    percent_progress:
      description: The percentage of the estimated data that has been transferred.
      returned: always
      type: int
      sample: 0
    port:
      description: The port that the DB cluster was listening on at the time of the snapshot.
      returned: always
      type: int
      sample: 0
    snapshot_create_time:
      description: The time when the snapshot was taken.
      returned: always
      type: str
      sample: "2018-05-17T00:23:23.731000+00:00"
    snapshot_type:
      description: The type of the DB cluster snapshot.
      returned: always
      type: str
      sample: "manual"
    status:
      description: The status of this DB cluster snapshot.
      returned: always
      type: str
      sample: "creating"
    storage_encrypted:
      description: Indicates whether the DB cluster snapshot is encrypted.
      returned: always
      type: bool
      sample: true
    tags:
      description: Tags of the snapshot.
      returned: when snapshot is not shared.
      type: complex
      contains: {}
    vpc_id:
      description: The VPC ID associated with the DB cluster snapshot.
      returned: always
      type: str
      sample: "vpc-abcd1234"
"""

from typing import Any
from typing import Callable
from typing import Dict
from typing import List

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.rds import AnsibleRDSError
from ansible_collections.amazon.aws.plugins.module_utils.rds import describe_db_cluster_snapshots
from ansible_collections.amazon.aws.plugins.module_utils.rds import describe_db_snapshots
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict


def common_snapshot_info(
    client, module: AnsibleAWSModule, describe_snapshots_method: Callable, params: Dict[str, Any]
) -> List[Dict[str, Any]]:
    try:
        results = describe_snapshots_method(client, **params)
    except AnsibleRDSError as e:
        module.fail_json_aws(e, "Failed to get snapshot information")

    for snapshot in results:
        if snapshot["SnapshotType"] != "shared":
            snapshot["Tags"] = boto3_tag_list_to_ansible_dict(snapshot.pop("Tags", []))

    return [camel_dict_to_snake_dict(snapshot, ignore_list=["Tags"]) for snapshot in results]


def cluster_snapshot_info(client, module: AnsibleAWSModule) -> List[Dict[str, Any]]:
    snapshot_id = module.params.get("db_cluster_snapshot_identifier")
    snapshot_type = module.params.get("snapshot_type")
    instance_name = module.params.get("db_cluster_identifier")

    params = dict()
    if snapshot_id:
        params["DBClusterSnapshotIdentifier"] = snapshot_id
    if instance_name:
        params["DBClusterIdentifier"] = instance_name
    if snapshot_type:
        params["SnapshotType"] = snapshot_type
        if snapshot_type == "public":
            params["IncludePublic"] = True
        elif snapshot_type == "shared":
            params["IncludeShared"] = True

    return common_snapshot_info(client, module, describe_db_cluster_snapshots, params)


def instance_snapshot_info(client, module: AnsibleAWSModule) -> List[Dict[str, Any]]:
    snapshot_id = module.params.get("db_snapshot_identifier")
    snapshot_type = module.params.get("snapshot_type")
    instance_name = module.params.get("db_instance_identifier")

    params = dict()
    if snapshot_id:
        params["DBSnapshotIdentifier"] = snapshot_id
    if instance_name:
        params["DBInstanceIdentifier"] = instance_name
    if snapshot_type:
        params["SnapshotType"] = snapshot_type
        if snapshot_type == "public":
            params["IncludePublic"] = True
        elif snapshot_type == "shared":
            params["IncludeShared"] = True

    return common_snapshot_info(client, module, describe_db_snapshots, params)


def main():
    argument_spec = dict(
        db_snapshot_identifier=dict(aliases=["snapshot_name"]),
        db_instance_identifier=dict(),
        db_cluster_identifier=dict(),
        db_cluster_snapshot_identifier=dict(),
        snapshot_type=dict(choices=["automated", "manual", "shared", "public"]),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[
            [
                "db_snapshot_identifier",
                "db_instance_identifier",
                "db_cluster_identifier",
                "db_cluster_snapshot_identifier",
            ]
        ],
    )

    client = module.client("rds")
    results = dict()
    if not module.params["db_cluster_identifier"] and not module.params["db_cluster_snapshot_identifier"]:
        results["snapshots"] = instance_snapshot_info(client, module)
    if not module.params["db_snapshot_identifier"] and not module.params["db_instance_identifier"]:
        results["cluster_snapshots"] = cluster_snapshot_info(client, module)

    module.exit_json(changed=False, **results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2020 Guopeng Lin, <linguopeng1998@google.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = '''
module: azure_rm_adapplication_info

version_added: "1.6.0"

short_description: Get Azure Active Directory application info

description:
    - Get Azure Active Directory application info.

options:
    app_id:
        description:
            - The application ID.
        type: str
    object_id:
        description:
            - The application's object ID.
        type: str
    identifier_uri:
        description:
            - The identifier_uri's object ID.
        type: str
    app_display_name:
        description:
            - The application Name.
        type: str
    app_diff:
        description:
            - A list of applications
            - The application name or application ID is mandatory
            - All fields of the applications can also be provided (parsing from a JSON or YAML).
            - With this option, you can compare your self-defined applications with the current state using an application list.\
              The applications that are present only in current state but not in your list will be returned as a list with all fields\
              and will receive an additional status of ABSENT.\
              This allows you to first add applications with azure_rm_adapplication and then use azure_rm_adapplication_info (option diff)\
              to identify the applications that should not be in the current state and subsequently delete them with azure_rm_adapplication
        type: list
        elements: dict

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    haiyuan_zhang (@haiyuazhang)
    Fred-sun (@Fred-sun)
    guopeng_lin (@guopenglin)
    Xu Zhang (@xuzhang)
'''

EXAMPLES = '''
- name: get ad app info by App ID
  azure_rm_adapplication_info:
    app_id: "{{ app_id }}"

- name: get ad app info ---- by object ID
  azure_rm_adapplication_info:
    object_id: "{{ object_id }}"

- name: get ad app info ---- by identifier uri
  azure_rm_adapplication_info:
    identifier_uri: "{{ identifier_uri }}"

- name: get ad app info ---- by display name
  azure_rm_adapplication_info:
    app_display_name: "{{ display_name }}"

- name: get ad app diff ---- by display name
  azure_rm_adapplication_info:
    app_diff:
      - app_display_name: "{{ display_name }}"
      - app_id: "{{ app_id }}"
'''

RETURN = '''
applications:
    description:
        - The info of the ad application.
    type: complex
    returned: aways
    contains:
        app_display_name:
            description:
                - Object's display name or its prefix.
            type: str
            returned: always
            sample: app
        app_id:
            description:
                - The application ID.
            returned: always
            type: str
            sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
        identifier_uris:
            description:
                - The identifiers_uri list of app.
            type: list
            returned: always
            sample: ["http://ansible-atodorov"]
        object_id:
            description:
                - It's application's object ID.
            returned: always
            type: str
            sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
        sign_in_audience:
            description:
                - The application can be used from any Azure AD tenants
            type: str
            returned: always
            sample: AzureADandPersonalMicrosoftAccount
        public_client_reply_urls:
            description:
                - The public client redirect urls.
                - Space-separated URIs to which Azure AD will redirect in response to an OAuth 2.0 request.
            returned: always
            type: list
            sample: []
        web_reply_urls:
            description:
                - The web redirect urls.
                - Space-separated URIs to which Azure AD will redirect in response to an OAuth 2.0 request.
            returned: always
            type: list
            sample: []
        spa_reply_urls:
            description:
                - The spa redirect urls.
                - Space-separated URIs to which Azure AD will redirect in response to an OAuth 2.0 request.
            returned: always
            type: list
            sample: []
        optional_claims:
            description:
                - Declare the optional claims for the application.
            type: complex
            returned: always
            contains:
                access_token_claims :
                    description:
                        - The optional claims returned in the JWT access token
                    type: list
                    returned: always
                    sample: ['name': 'aud', 'source': null, 'essential': false, 'additional_properties': []]
                id_token_claims:
                    description:
                        - The optional claims returned in the JWT ID token
                    type: list
                    returned: always
                    sample: ['name': 'acct', 'source': null, 'essential': false, 'additional_properties': []]
                saml2_token_claims:
                    description:
                        - The optional claims returned in the SAML token
                    type: list
                    returned: always
                    sample: ['name': 'acct', 'source': null, 'essential': false, 'additional_properties': []]
app_diff:
    description:
        - The info of the ad application.
    type: complex
    returned: aways
    contains:
        app_display_name:
            description:
                - Object's display name or its prefix.
            type: str
            returned: always
            sample: app
        app_id:
            description:
                - The application ID.
            returned: always
            type: str
            sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
        identifier_uris:
            description:
                - The identifiers_uri list of app.
            type: list
            returned: always
            sample: ["http://ansible-atodorov"]
        object_id:
            description:
                - It's application's object ID.
            returned: always
            type: str
            sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
        sign_in_audience:
            description:
                - The application can be used from any Azure AD tenants
            type: str
            returned: always
            sample: AzureADandPersonalMicrosoftAccount
        public_client_reply_urls:
            description:
                - The public client redirect urls.
                - Space-separated URIs to which Azure AD will redirect in response to an OAuth 2.0 request.
            returned: always
            type: list
            sample: []
        web_reply_urls:
            description:
                - The web redirect urls.
                - Space-separated URIs to which Azure AD will redirect in response to an OAuth 2.0 request.
            returned: always
            type: list
            sample: []
        spa_reply_urls:
            description:
                - The spa redirect urls.
                - Space-separated URIs to which Azure AD will redirect in response to an OAuth 2.0 request.
            returned: always
            type: list
            sample: []
        state:
            description:
                - absent --> The app isn't in the app_diff.
            returned: always
            type: str
            sample: absent
        optional_claims:
            description:
                - Declare the optional claims for the application.
            type: complex
            returned: always
            contains:
                access_token_claims :
                    description:
                        - The optional claims returned in the JWT access token
                    type: list
                    returned: always
                    sample: ['name': 'aud', 'source': null, 'essential': false, 'additional_properties': []]
                id_token_claims:
                    description:
                        - The optional claims returned in the JWT ID token
                    type: list
                    returned: always
                    sample: ['name': 'acct', 'source': null, 'essential': false, 'additional_properties': []]
                saml2_token_claims:
                    description:
                        - The optional claims returned in the SAML token
                    type: list
                    returned: always
                    sample: ['name': 'acct', 'source': null, 'essential': false, 'additional_properties': []]
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBase

try:
    import asyncio
    from msgraph.generated.applications.applications_request_builder import ApplicationsRequestBuilder
    from kiota_abstractions.api_error import APIError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMADApplicationInfo(AzureRMModuleBase):

    def __init__(self):
        self.module_arg_spec = dict(
            app_id=dict(type='str'),
            object_id=dict(type='str'),
            identifier_uri=dict(type='str'),
            app_display_name=dict(type='str'),
            app_diff=dict(type='list', elements='dict')
        )
        self.app_id = None
        self.app_display_name = None
        self.object_id = None
        self.identifier_uri = None
        self.results = dict(changed=False)
        self._client = None
        super(AzureRMADApplicationInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                       supports_check_mode=True,
                                                       supports_tags=False,
                                                       is_ad_resource=True)

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        applications = []
        try:
            self._client = self.get_msgraph_client()
            if self.object_id:
                applications = [asyncio.get_event_loop().run_until_complete(self.get_application(self.object_id))]
            else:
                sub_filters = []
                if self.identifier_uri:
                    sub_filters.append("identifierUris/any(s:s eq '{0}')".format(self.identifier_uri))
                if self.app_id:
                    sub_filters.append("appId eq '{0}'".format(self.app_id))
                if self.app_display_name:
                    sub_filters.append("displayName eq '{0}'".format(self.app_display_name))
                apps = asyncio.get_event_loop().run_until_complete(self.get_applications(sub_filters))
                applications = list(apps)
            self.results['applications'] = [self.to_dict(app) for app in applications]
            current_app = [self.to_dict(app) for app in applications]
        except APIError as e:
            if e.response_status_code != 404:
                self.fail("failed to get application info {0}".format(str(e)))
        except Exception as e:
            if "Name or service not known" in str(e):
                self.fail("DNS resolution error occurred.")
            else:
                self.fail("An unexpected error occurred: {0}".format(str(e)))
        if self.app_diff:
            temp_app = []
            for app in current_app:
                found = False
                for diff in self.app_diff:
                    if app.get("app_id") == diff.get("app_id") or app.get("app_display_name") == diff.get("app_display_name"):
                        found = True
                        break
                if not found:
                    app['state'] = 'absent'
                    temp_app.append(app)
            self.results['app_diff'] = temp_app
        return self.results

    def serialize_claims(self, claims):
        if claims is None:
            return None
        return [{
            "additional_properties": claim.additional_properties,
            "essential": claim.essential,
            "name": claim.name,
            "source": claim.source} for claim in claims]

    def to_dict(self, object):
        response = dict(
            app_id=object.app_id,
            object_id=object.id,
            app_display_name=object.display_name,
            identifier_uris=object.identifier_uris,
            sign_in_audience=object.sign_in_audience,
            web_reply_urls=object.web.redirect_uris,
            spa_reply_urls=object.spa.redirect_uris,
            public_client_reply_urls=object.public_client.redirect_uris,
            optional_claims=dict(access_token=[], id_token=[], saml2_token=[])
        )

        if object.optional_claims is not None:
            response['optional_claims']['id_token'] = self.serialize_claims(object.optional_claims.id_token)
            response['optional_claims']['saml2_token'] = self.serialize_claims(object.optional_claims.saml2_token)
            response['optional_claims']['access_token'] = self.serialize_claims(object.optional_claims.access_token)
        return response

    async def get_application(self, obj_id):
        return await self._client.applications.by_application_id(obj_id).get()

    async def get_applications(self, sub_filters):
        if sub_filters:
            request_configuration = ApplicationsRequestBuilder.ApplicationsRequestBuilderGetRequestConfiguration(
                query_parameters=ApplicationsRequestBuilder.ApplicationsRequestBuilderGetQueryParameters(
                    filter=(' and '.join(sub_filters)),
                ),
            )
            applications = await self._client.applications.get(request_configuration=request_configuration)
            return applications.value
        else:
            applications_list = []
            applications = await self._client.applications.get()
            for app in applications.value:
                applications_list.append(app)

            if applications.odata_next_link:
                next_link = applications.odata_next_link
            else:
                next_link = None

            while next_link:
                applications = await self._client.applications.with_url(next_link).get()
                next_link = applications.odata_next_link
                for app in applications.value:
                    applications_list.append(app)
            return applications_list


def main():
    AzureRMADApplicationInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: user
short_description: Resource module for User
description:
- This module represents an alias of the module user_v1
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  email:
    description: Email.
    type: str
  firstName:
    description: First Name.
    type: str
  lastName:
    description: Last Name.
    type: str
  password:
    description: Password.
    type: str
  roleList:
    description: Role id list.
    elements: str
    type: list
  userId:
    description: User Id.
    type: str
  username:
    description: Username.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for User and Roles AddUserAPIV1
  description: Complete reference of the AddUserAPIV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!add-user-api
- name: Cisco DNA Center documentation for User and Roles DeleteUserAPIV1
  description: Complete reference of the DeleteUserAPIV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-user-api
- name: Cisco DNA Center documentation for User and Roles UpdateUserAPIV1
  description: Complete reference of the UpdateUserAPIV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-user-api
notes:
  - SDK Method used are
    user_and_roles.UserandRoles.add_user_api_v1,
    user_and_roles.UserandRoles.delete_user_api_v1,
    user_and_roles.UserandRoles.update_user_api_v1,

  - Paths used are
    post /dna/system/api/v1/user,
    delete /dna/system/api/v1/user/{userId},
    put /dna/system/api/v1/user,
  - It should be noted that this module is an alias of user_v1

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.user:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    email: string
    firstName: string
    lastName: string
    password: string
    roleList:
    - string
    username: string

- name: Update all
  cisco.dnac.user:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    email: string
    firstName: string
    lastName: string
    roleList:
    - string
    userId: string
    username: string

- name: Delete by id
  cisco.dnac.user:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    userId: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "message": "string",
      "userId": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sda_transit_networks_v1_info
short_description: Information module for Sda Transit Networks V1
description:
- Get all Sda Transit Networks V1.
- Returns a list of transit networks that match the provided query parameters.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  id:
    description:
    - Id query parameter. ID of the transit network.
    type: str
  name:
    description:
    - Name query parameter. Name of the transit network.
    type: str
  type:
    description:
    - >
      Type query parameter. Type of the transit network. Allowed values are IP_BASED_TRANSIT,
      SDA_LISP_PUB_SUB_TRANSIT, SDA_LISP_BGP_TRANSIT.
    type: str
  offset:
    description:
    - Offset query parameter. Starting record for pagination.
    type: float
  limit:
    description:
    - Limit query parameter. Maximum number of records to return.
    type: float
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for SDA GetTransitNetworksV1
  description: Complete reference of the GetTransitNetworksV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-transit-networks
notes:
  - SDK Method used are
    sda.Sda.get_transit_networks_v1,

  - Paths used are
    get /dna/intent/api/v1/sda/transitNetworks,

"""

EXAMPLES = r"""
- name: Get all Sda Transit Networks V1
  cisco.dnac.sda_transit_networks_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
    name: string
    type: string
    offset: 0
    limit: 0
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "id": "string",
          "name": "string",
          "type": "string",
          "ipTransitSettings": {
            "routingProtocolName": "string",
            "autonomousSystemNumber": "string"
          },
          "sdaTransitSettings": {
            "isMulticastOverTransitEnabled": true,
            "controlPlaneNetworkDeviceIds": [
              "string"
            ]
          }
        }
      ],
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: assurance_events_count_info
short_description: Information module for Assurance Events Count Info
description:
- This module represents an alias of the module assurance_events_count_v1_info
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  deviceFamily:
    description:
    - >
      DeviceFamily query parameter. Device family. Please note that multiple families across network device type
      and client type is not allowed. For example, choosing `Routers` along with `Wireless Client` or `Unified AP`
      is not supported. Examples `deviceFamily=Switches and Hubs` (single deviceFamily requested)
      `deviceFamily=Switches and Hubs&deviceFamily=Routers` (multiple deviceFamily requested).
    type: str
  startTime:
    description:
    - >
      StartTime query parameter. Start time from which API queries the data set related to the resource. It must
      be specified in UNIX epochtime in milliseconds. Value is inclusive. If `startTime` is not provided, API will
      default to current time minus 24 hours.
    type: str
  endTime:
    description:
    - >
      EndTime query parameter. End time to which API queries the data set related to the resource. It must be
      specified in UNIX epochtime in milliseconds. Value is inclusive. If `endTime` is not provided, API will
      default to current time.
    type: str
  messageType:
    description:
    - >
      MessageType query parameter. Message type for the event. Examples `messageType=Syslog` (single messageType
      requested) `messageType=Trap&messageType=Syslog` (multiple messageType requested).
    type: str
  severity:
    description:
    - >
      Severity query parameter. Severity of the event between 0 and 6. This is applicable only for events related
      to network devices (other than AP) and `Wired Client` events. | Value | Severity | | ----- | ----------- | |
      0 | Emergency | | 1 | Alert | | 2 | Critical | | 3 | Error | | 4 | Warning | | 5 | Notice | | 6 | Info |
      Examples `severity=0` (single severity requested) `severity=0&severity=1` (multiple severity requested).
    type: str
  siteId:
    description:
    - >
      SiteId query parameter. The UUID of the site. (Ex. `flooruuid`) Examples `?siteId=id1` (single siteId
      requested) `?siteId=id1&siteId=id2&siteId=id3` (multiple siteId requested).
    type: str
  siteHierarchyId:
    description:
    - >
      SiteHierarchyId query parameter. The full hierarchy breakdown of the site tree in id form starting from
      Global site UUID and ending with the specific site UUID. (Ex. `globalUuid/areaUuid/buildingUuid/floorUuid`)
      This field supports wildcard asterisk (`*`) character search support. E.g. `*uuid*, *uuid, uuid*` Examples
      `?siteHierarchyId=globalUuid/areaUuid/buildingUuid/floorUuid `(single siteHierarchyId requested) `?siteHiera
      rchyId=globalUuid/areaUuid/buildingUuid/floorUuid&siteHierarchyId=globalUuid/areaUuid2/buildingUuid2/floorUu
      id2` (multiple siteHierarchyId requested).
    type: str
  networkDeviceName:
    description:
    - >
      NetworkDeviceName query parameter. Network device name. This parameter is applicable for network device
      related families. This field supports wildcard (`*`) character-based search. Ex `*Branch*` or `Branch*` or
      `*Branch` Examples `networkDeviceName=Branch-3-Gateway` (single networkDeviceName requested)
      `networkDeviceName=Branch-3-Gateway&networkDeviceName=Branch-3-Switch` (multiple networkDeviceName
      requested).
    type: str
  networkDeviceId:
    description:
    - >
      NetworkDeviceId query parameter. The list of Network Device Uuids. (Ex.
      `6bef213c-19ca-4170-8375-b694e251101c`) Examples `networkDeviceId=6bef213c-19ca-4170-8375-b694e251101c`
      (single networkDeviceId requested) `networkDeviceId=6bef213c-19ca-4170-8375-
      b694e251101c&networkDeviceId=32219612-819e-4b5e-a96b-cf22aca13dd9&networkDeviceId=2541e9a7-b80d-4955-8aa2-
      79b233318ba0` (multiple networkDeviceId requested).
    type: str
  apMac:
    description:
    - >
      ApMac query parameter. MAC address of the access point. This parameter is applicable for `Unified AP` and
      `Wireless Client` events. This field supports wildcard (`*`) character-based search. Ex `*50 0F*` or `50
      0F*` or `*50 0F` Examples `apMac=50 0F 80 0F F7 E0` (single apMac requested) `apMac=50 0F 80 0F F7
      E0&apMac=18 80 90 AB 7E A0` (multiple apMac requested).
    type: str
  clientMac:
    description:
    - >
      ClientMac query parameter. MAC address of the client. This parameter is applicable for `Wired Client` and
      `Wireless Client` events. This field supports wildcard (`*`) character-based search. Ex `*66 2B*` or `66
      2B*` or `*66 2B` Examples `clientMac=66 2B B8 D2 01 56` (single clientMac requested) `clientMac=66 2B B8 D2
      01 56&clientMac=DC A6 32 F5 5A 89` (multiple clientMac requested).
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices CountTheNumberOfEventsV1
  description: Complete reference of the CountTheNumberOfEventsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!count-the-number-of-events
notes:
  - SDK Method used are
    devices.Devices.count_the_number_of_events_v1,

  - Paths used are
    get /dna/data/api/v1/assuranceEvents/count,
  - It should be noted that this module is an alias of assurance_events_count_v1_info

"""

EXAMPLES = r"""
- name: Get all Assurance Events Count Info
  cisco.dnac.assurance_events_count_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    deviceFamily: string
    startTime: string
    endTime: string
    messageType: string
    severity: string
    siteId: string
    siteHierarchyId: string
    networkDeviceName: string
    networkDeviceId: string
    apMac: string
    clientMac: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "count": 0
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: guest_type
short_description: Resource module for Guest Type
description:
- Manage operations create, update and delete of the resource Guest Type.
- This API creates a guest type.
- This API deletes a guest type.
- This API allows the client to update a guest type.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  accessTime:
    description: Guest Type's accessTime.
    suboptions:
      allowAccessOnSpecificDaysTimes:
        description: AllowAccessOnSpecificDaysTimes flag.
        type: bool
      dayTimeLimits:
        description: List of Time Ranges for account access.
        elements: dict
        suboptions:
          days:
            description: List of Days Values should be one of Week day. Allowed values
              are - Sunday, - Monday, - Tuesday, - Wednesday, - Thursday, - Friday,
              - Saturday.
            elements: str
            type: list
          endTime:
            description: End time in HH mm format.
            type: str
          startTime:
            description: Start time in HH mm format.
            type: str
        type: list
      defaultDuration:
        description: Guest Type's defaultDuration.
        type: int
      durationTimeUnit:
        description: Allowed values are - DAYS, - HOURS, - MINUTES.
        type: str
      fromFirstLogin:
        description: When Account Duration starts from first login or specified date.
        type: bool
      maxAccountDuration:
        description: Maximum value of Account Duration.
        type: int
    type: dict
  description:
    description: Guest Type's description.
    type: str
  expirationNotification:
    description: Expiration Notification Settings.
    suboptions:
      advanceNotificationDuration:
        description: Send Account Expiration Notification Duration before ( Days, Hours,
          Minutes ).
        type: int
      advanceNotificationUnits:
        description: Allowed values are - DAYS, - HOURS, - MINUTES.
        type: str
      emailText:
        description: Guest Type's emailText.
        type: str
      enableNotification:
        description: Enable Notification settings.
        type: bool
      sendEmailNotification:
        description: Enable Email Notification.
        type: bool
      sendSMSNotification:
        description: Maximum devices guests can register.
        type: bool
      smsText:
        description: Guest Type's smsText.
        type: str
    type: dict
  id:
    description: Guest Type's id.
    type: str
  isDefaultType:
    description: IsDefaultType flag.
    type: bool
  loginOptions:
    description: Guest Type's loginOptions.
    suboptions:
      allowGuestPortalBypass:
        description: AllowGuestPortalBypass flag.
        type: bool
      failureAction:
        description: When Guest Exceeds limit this action will be invoked. Allowed values
          are - Disconnect_Oldest_Connection, - Disconnect_Newest_Connection.
        type: str
      identityGroupId:
        description: Guest Type's identityGroupId.
        type: str
      limitSimultaneousLogins:
        description: Enable Simultaneous Logins.
        type: bool
      maxRegisteredDevices:
        description: Maximum devices guests can register.
        type: int
      maxSimultaneousLogins:
        description: Number of Simultaneous Logins.
        type: int
    type: dict
  name:
    description: Guest Type's name.
    type: str
  sponsorGroups:
    description: Guest Type's sponsorGroups.
    elements: str
    type: list
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    guest_type.GuestType.create_guest_type,
    guest_type.GuestType.delete_guest_type_by_id,
    guest_type.GuestType.update_guest_type_by_id,

  - Paths used are
    post /ers/config/guesttype,
    delete /ers/config/guesttype/{id},
    put /ers/config/guesttype/{id},

"""

EXAMPLES = r"""
- name: Update by id
  cisco.ise.guest_type:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    accessTime:
      allowAccessOnSpecificDaysTimes: true
      dayTimeLimits:
      - days:
        - string
        endTime: string
        startTime: string
      defaultDuration: 0
      durationTimeUnit: string
      fromFirstLogin: true
      maxAccountDuration: 0
    description: string
    expirationNotification:
      advanceNotificationDuration: 0
      advanceNotificationUnits: string
      emailText: string
      enableNotification: true
      sendEmailNotification: true
      sendSmsNotification: true
      smsText: string
    id: string
    isDefaultType: true
    loginOptions:
      allowGuestPortalBypass: true
      failureAction: string
      identityGroupId: string
      limitSimultaneousLogins: true
      maxRegisteredDevices: 0
      maxSimultaneousLogins: 0
    name: string
    sponsorGroups:
    - string

- name: Delete by id
  cisco.ise.guest_type:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: absent
    id: string

- name: Create
  cisco.ise.guest_type:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    accessTime:
      allowAccessOnSpecificDaysTimes: true
      dayTimeLimits:
      - days:
        - string
        endTime: string
        startTime: string
      defaultDuration: 0
      durationTimeUnit: string
      fromFirstLogin: true
      maxAccountDuration: 0
    description: string
    expirationNotification:
      advanceNotificationDuration: 0
      advanceNotificationUnits: string
      emailText: string
      enableNotification: true
      sendEmailNotification: true
      sendSmsNotification: true
      smsText: string
    isDefaultType: true
    loginOptions:
      allowGuestPortalBypass: true
      failureAction: string
      identityGroupId: string
      limitSimultaneousLogins: true
      maxRegisteredDevices: 0
      maxSimultaneousLogins: 0
    name: string
    sponsorGroups:
    - string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "name": "string",
      "description": "string",
      "isDefaultType": true,
      "accessTime": {
        "fromFirstLogin": true,
        "maxAccountDuration": 0,
        "durationTimeUnit": "string",
        "defaultDuration": 0,
        "allowAccessOnSpecificDaysTimes": true,
        "dayTimeLimits": [
          {
            "startTime": "string",
            "endTime": "string",
            "days": [
              "string"
            ]
          }
        ]
      },
      "loginOptions": {
        "limitSimultaneousLogins": true,
        "maxSimultaneousLogins": 0,
        "failureAction": "string",
        "maxRegisteredDevices": 0,
        "identityGroupId": "string",
        "allowGuestPortalBypass": true
      },
      "expirationNotification": {
        "enableNotification": true,
        "advanceNotificationDuration": 0,
        "advanceNotificationUnits": "string",
        "sendEmailNotification": true,
        "emailText": "string",
        "sendSmsNotification": true,
        "smsText": "string"
      },
      "sponsorGroups": [
        "string"
      ],
      "link": {
        "rel": "string",
        "href": "string",
        "type": "string"
      }
    }

ise_update_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  version_added: '1.1.0'
  type: dict
  sample: >
    {
      "UpdatedFieldsList": {
        "updatedField": [
          {
            "field": "string",
            "oldValue": "string",
            "newValue": "string"
          }
        ],
        "field": "string",
        "oldValue": "string",
        "newValue": "string"
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: tacacs_external_servers
short_description: Resource module for TACACS External Servers
description:
- Manage operations create, update and delete of the resource TACACS External Servers.
- This API creates TACACS external servers.
- This API deletes TACACS external servers.
- This API allows the client to update TACACS external servers.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  connectionPort:
    description: The port to connect the server.
    type: int
  description:
    description: TACACS External Servers's description.
    type: str
  hostIP:
    description: The server IPV4 address.
    type: str
  id:
    description: TACACS External Servers's id.
    type: str
  name:
    description: TACACS External Servers's name.
    type: str
  sharedSecret:
    description: The server shared secret.
    type: str
  singleConnect:
    description: Define the use of single connection.
    type: bool
  timeout:
    description: The server timeout.
    type: int
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    tacacs_external_servers.TacacsExternalServers.create_tacacs_external_servers,
    tacacs_external_servers.TacacsExternalServers.delete_tacacs_external_servers_by_id,
    tacacs_external_servers.TacacsExternalServers.update_tacacs_external_servers_by_id,

  - Paths used are
    post /ers/config/tacacsexternalservers,
    delete /ers/config/tacacsexternalservers/{id},
    put /ers/config/tacacsexternalservers/{id},

"""

EXAMPLES = r"""
- name: Update by id
  cisco.ise.tacacs_external_servers:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    connectionPort: 0
    description: string
    hostIP: string
    id: string
    name: string
    sharedSecret: string
    singleConnect: true
    timeout: 0

- name: Delete by id
  cisco.ise.tacacs_external_servers:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: absent
    id: string

- name: Create
  cisco.ise.tacacs_external_servers:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    connectionPort: 0
    description: string
    hostIP: string
    name: string
    sharedSecret: string
    singleConnect: true
    timeout: 0

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "name": "string",
      "description": "string",
      "hostIP": "string",
      "connectionPort": 0,
      "singleConnect": true,
      "sharedSecret": "string",
      "timeout": 0,
      "link": {
        "rel": "string",
        "href": "string",
        "type": "string"
      }
    }

ise_update_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  version_added: '1.1.0'
  type: dict
  sample: >
    {
      "UpdatedFieldsList": {
        "updatedField": [
          {
            "field": "string",
            "oldValue": "string",
            "newValue": "string"
          }
        ],
        "field": "string",
        "oldValue": "string",
        "newValue": "string"
      }
    }
"""



#
# (c) 2016 Red Hat Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type

import json
import re

from ansible.errors import AnsibleConnectionFailure
from ansible.module_utils._text import to_bytes, to_text
from ansible.utils.display import Display
from ansible_collections.ansible.netcommon.plugins.plugin_utils.terminal_base import TerminalBase


display = Display()


class TerminalModule(TerminalBase):
    terminal_stdout_re = [
        re.compile(rb"[\r\n]?[\w\+\-\.:\/\[\]]+(?:\([^\)]+\)){0,3}(?:[>#]) ?$"),
    ]

    privilege_level_re = re.compile(r"Current privilege level is (\d+)$")

    terminal_stderr_re = [
        re.compile(rb"% ?Error"),
        # re.compile(rb"^% \w+", re.M),
        re.compile(rb"ERROR:", re.IGNORECASE),
        re.compile(rb"% ?Bad secret"),
        re.compile(rb"[\r\n%] Bad passwords"),
        re.compile(rb"invalid input", re.I),
        re.compile(rb"(?:incomplete|ambiguous) command", re.I),
        re.compile(rb"connection timed out", re.I),
        re.compile(rb"[^\r\n]+ not found"),
        re.compile(rb"'[^']' +returned error code: ?\d+"),
        re.compile(rb"Bad mask", re.I),
        re.compile(rb"% ?(\S+) ?overlaps with ?(\S+)", re.I),
        re.compile(rb"% ?(\S+) ?Error: ?[\s]+", re.I),
        re.compile(rb"% ?(\S+) ?Informational: ?[\s]+", re.I),
        re.compile(rb"Command authorization failed"),
        re.compile(rb"Command Rejected: ?[\s]+", re.I),
        re.compile(
            rb"% General session commands not allowed under the address family",
            re.I,
        ),
        re.compile(rb"% BGP: Error initializing topology", re.I),
        re.compile(rb"%SNMP agent not enabled", re.I),
        re.compile(rb"% Invalid", re.I),
        re.compile(
            rb"%You must disable VTPv1 and VTPv2 or switch to VTPv3 before configuring a VLAN name longer than 32 characters",
            re.I,
        ),
    ]

    terminal_config_prompt = re.compile(r"^.+\(config(-.*)?\)#$")

    def get_privilege_level(self):
        try:
            cmd = {"command": "show privilege"}
            result = self._exec_cli_command(
                to_bytes(json.dumps(cmd), errors="surrogate_or_strict"),
            )
        except AnsibleConnectionFailure as e:
            raise AnsibleConnectionFailure(
                "unable to fetch privilege, with error: %s" % (e.message),
            )

        prompt = self.privilege_level_re.search(result)
        if not prompt:
            raise AnsibleConnectionFailure(
                "unable to check privilege level [%s]" % result,
            )

        return int(prompt.group(1))

    def on_open_shell(self):
        _is_sdWan = False  # initialize to false for default IOS execution
        try:
            self._exec_cli_command(b"terminal length 0")
        except AnsibleConnectionFailure:
            try:
                self._exec_cli_command(b"screen-length 0")  # support to SD-WAN mode
                _is_sdWan = True
            except AnsibleConnectionFailure:  # fails as length required for handling prompt
                raise AnsibleConnectionFailure("unable to set terminal parameters")
        try:
            if _is_sdWan:
                self._exec_cli_command(b"screen-width 512")  # support to SD-WAN mode
            else:
                self._exec_cli_command(b"terminal width 512")
                try:
                    self._exec_cli_command(b"terminal width 0")
                except AnsibleConnectionFailure:
                    pass
        except AnsibleConnectionFailure:
            display.display(
                "WARNING: Unable to set terminal/screen width, command responses may be truncated",
            )

    def on_become(self, passwd=None):
        if self._get_prompt().endswith(b"#") and self.get_privilege_level() == 15:
            return

        cmd = {"command": "enable"}
        if passwd:
            # Note: python-3.5 cannot combine u"" and r"" together.  Thus make
            # an r string and use to_text to ensure it's text on both py2 and py3.
            cmd["prompt"] = to_text(
                r"[\r\n]?(?:.*)?[Pp]assword: ?$",
                errors="surrogate_or_strict",
            )
            cmd["answer"] = passwd
            cmd["prompt_retry_check"] = True
        try:
            self._exec_cli_command(
                to_bytes(json.dumps(cmd), errors="surrogate_or_strict"),
            )
            prompt = self._get_prompt()
            privilege_level = self.get_privilege_level()
        except AnsibleConnectionFailure as e:
            prompt = self._get_prompt()
            raise AnsibleConnectionFailure(
                "failed to elevate privilege to enable mode, at prompt [%s] with error: %s"
                % (prompt, e.message),
            )

        if prompt is None or not prompt.endswith(b"#") or privilege_level != 15:
            raise AnsibleConnectionFailure(
                "failed to elevate privilege to enable mode, still at level [%d] and prompt [%s]"
                % (privilege_level, prompt),
            )

    def on_unbecome(self):
        prompt = self._get_prompt()
        if prompt is None:
            # if prompt is None most likely the terminal is hung up at a prompt
            return

        if self.get_privilege_level() != 15:
            return

        if b"(config" in prompt:
            self._exec_cli_command(b"end")
            self._exec_cli_command(b"disable")

        elif prompt.endswith(b"#"):
            self._exec_cli_command(b"disable")
